# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.contrib import admin
from .models import Log, APILog, File
# Register your models here.


class LogAdmin(admin.ModelAdmin):
    pass


class APILogAdmin(admin.ModelAdmin):
    pass


class FileAdmin(admin.ModelAdmin):
    pass


admin.site.register(Log, LogAdmin)
admin.site.register(APILog, APILogAdmin)
admin.site.register(File, FileAdmin)



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: group.py
# Project: django-enterprise-core
# File Created: Wednesday, 22nd August 2018 12:45:28 am
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Wednesday, 22nd August 2018 12:45:28 am
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-enterprise-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.views.generic import TemplateView
from django.shortcuts import redirect
from django.contrib import messages
from django.contrib.auth.models import Group
from django.utils.translation import gettext_lazy as _
from enterprise.libs.view import ProtectedMixin
from datatable import Datatable
from ..forms import GroupForm


class GroupView(ProtectedMixin, TemplateView):
    template_name = "superuser/group.html"

    def get(self, request, *args, **kwargs):

        if request.GET.get('draw', None) is not None:
            return self.datatable(request)

        return self.render_to_response({
            'title': "Tracker enterprise - %s" % Group.__name__,
        })

    def delete(self, request):
        o_id = request.body.decode('utf-8').split("=")[1]
        qs = Group.objects.filter(id__exact=o_id).first()
        qs.delete()
        return self.render_to_response({})

    def datatable(self, request):
        def get_permission_list(self):
            return " | ".join(self.permissions.values_list("name", flat=True))

        Group.add_to_class("get_permission_list", get_permission_list)
        qs = Group.objects.all()
        defer = ['id', 'name', 'get_permission_list']

        d = Datatable(request, qs, defer, key="id")
        return d.get_data()


class GroupFormView(ProtectedMixin, TemplateView):
    template_name = "superuser/group.form.html"

    def get(self, request, *args, **kwargs):
        edit = request.GET.get("edit")

        if edit:
            form = GroupForm(instance=Group.objects.get(id=edit))
        else:
            form = GroupForm()

        return self.render_to_response({"form": form})

    def post(self, request):
        edit = request.GET.get("edit")

        if edit:
            form = GroupForm(request.POST, instance=Group.objects.get(id=edit))
        else:
            form = GroupForm(request.POST)

        if form.is_valid():
            group = form.save(commit=False)
            group.save()
            group.permissions.set(form.cleaned_data.get('permissions'))
            group.save()
            messages.success(
                request, _(
                    'Group (%s) has been saved.' %
                    group.name))
            return redirect("superuser:group")
        else:
            return self.render_to_response({"form": form})



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: xendit.py
# Project: core.wecare.id
# File Created: Saturday, 27th April 2019 1:23:38 pm
# 
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
# 
# Last Modified: Saturday, 27th April 2019 1:23:38 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
# 
# Handcrafted and Made with Love
# Copyright - 2018 Wecare.Id, wecare.id
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


import requests
import base64
import json
import ssl
import datetime

from django.utils.translation import gettext_lazy as _
from django.conf import settings


class Xendit(object):

    disbursement = None
    bank_account = None
    def __init__(self, disbursement, bank_account):
        self.disbursement = disbursement
        self.bank_account = bank_account

    def disburse(self):
        headers = {
            'X-IDEMPOTENCY-KEY' : self.disbursement.id62
        }
        data = {
            'external_id' : self.disbursement.nonce,
            'bank_code' : self.bank_account.bank.display_name,
            'account_holder_name' : self.bank_account.name,
            'account_number' : self.bank_account.number,
            'description' : self.disbursement.description,
            'amount' : self.disbursement.amount
        }
        r = requests.post(
            'https://api.xendit.co/disbursements', 
            headers=headers, 
            data=data, 
            auth=(getattr(settings, 'XENDIT_SECRET_KEY'), '')
        )
        r_dict = r.json()

        return r_dict



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: storage.py
# Project: core.ayopeduli.id
# File Created: Wednesday, 31st October 2018 7:26:00 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Wednesday, 31st October 2018 7:26:01 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Peduli sesama, sejahtera bersama
# Copyright - 2018 Ayopeduli.Id, ayopeduli.id
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.conf import settings
from django.core.files.storage import FileSystemStorage
from enterprise.libs.rackspace_cloud_files import RackspaceStorage


def generate_name(instance, filename):
    pass


if settings.PRODUCTION:
    ROOT_URL = ""
    MEDIA_ROOT = "/var/www/html/upload/"
    UPLOAD_ROOT = "%supload/" % settings.BASE_URL
else:
    ROOT_URL = "dev/"
    MEDIA_ROOT = settings.MEDIA_ROOT
    UPLOAD_ROOT = "%sstatic/upload/" % settings.BASE_URL


if settings.USE_RACKSPACE:
    BASE_URL = settings.RACKSPACE_BASE_URL
    VIDEO_STORAGE = RackspaceStorage(
        location="%svideo/" % ROOT_URL,
        base_url=BASE_URL,
        purpose="VIDEO_SIZES"
    )

    FILE_STORAGE = RackspaceStorage(
        location="%sfile/" % ROOT_URL,
        base_url=BASE_URL,
        purpose="FILE"
    )

    AVATAR_STORAGE = RackspaceStorage(
        location="%spicture/avatar/" % ROOT_URL,
        base_url=BASE_URL,
        purpose="AVATAR_PHOTO_SIZES"
    )

    COVER_STORAGE = RackspaceStorage(
        location="%spicture/cover/" % ROOT_URL,
        base_url=BASE_URL,
        purpose="COVER_PHOTO_SIZES"
    )

    LOGO_STORAGE = RackspaceStorage(
        location="%spicture/logo/" % ROOT_URL,
        base_url=BASE_URL,
        purpose="LOGO_PHOTO_SIZES"
    )

    PICTURE_STORAGE = RackspaceStorage(
        location="%spicture/others/" % ROOT_URL,
        base_url=BASE_URL,
        purpose="PICTURE_PHOTO_SIZES"
    )

else:

    VIDEO_STORAGE = FileSystemStorage(
        location='%s/video' %
        MEDIA_ROOT,
        base_url='%svideo/' %
        UPLOAD_ROOT)
    FILE_STORAGE = FileSystemStorage(
        location='%s/file' %
        MEDIA_ROOT,
        base_url='%sfile/' %
        UPLOAD_ROOT)
    AVATAR_STORAGE = FileSystemStorage(
        location='%s/picture/avatar' %
        MEDIA_ROOT,
        base_url='%spicture/avatar/' %
        UPLOAD_ROOT)
    COVER_STORAGE = FileSystemStorage(
        location='%s/picture/cover' %
        MEDIA_ROOT,
        base_url='%spicture/cover/' %
        UPLOAD_ROOT)
    LOGO_STORAGE = FileSystemStorage(
        location='%s/picture/logo' %
        MEDIA_ROOT,
        base_url='%spicture/logo/' %
        UPLOAD_ROOT)
    PICTURE_STORAGE = FileSystemStorage(
        location='%s/picture/others' %
        MEDIA_ROOT,
        base_url='%spicture/others/' %
        UPLOAD_ROOT)



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: urls.py
# Project: <<projectname>>
# File Created: Wednesday, 20th February 2019 2:54:10 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         http://ardz.xyz>
#
# Last Modified: Wednesday, 20th February 2019 2:54:10 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Crafted by Pro
# Copyright - <<year>> Ardz & Co, -
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.conf.urls import url, include
from .views import IndexView
from .views.raw import RawView

urlpatterns = [
    url(r'^raw/$', RawView.as_view(), name='raw'),
    url(r'^$', IndexView.as_view(), name='index'),
]



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: json_response.py
# Project: django-enterprise-core
# File Created: Wednesday, 5th September 2018 12:02:51 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Wednesday, 5th September 2018 12:02:51 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-enterprise-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.http import HttpResponse
from rest_framework.renderers import JSONRenderer


class JSONResponse(HttpResponse):

    def __init__(self, data, **kwargs):
        content = JSONRenderer().render(data)
        kwargs['content_type'] = 'application/json'
        super(JSONResponse, self).__init__(content, **kwargs)



# users/admin.py
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from django.utils.translation import gettext_lazy as _

from .forms import CustomUserCreationForm, CustomUserChangeForm
from .models import User


class CustomUserAdmin(UserAdmin):
    add_form = CustomUserCreationForm
    form = CustomUserChangeForm
    model = User
    list_display = ['phone_number', 'id62', 'full_name', 'email', ]
    fieldsets = (
        (None, {'fields': ('phone_number', 'password')}),
        (_('Personal info'), {'fields': ('full_name', 'nick_name', 'email')}),
        (_('Permissions'), {'fields': ('is_active', 'is_staff', 'is_superuser',
                                       'groups', 'user_permissions')}),
        (_('Important dates'), {'fields': ('last_login', 'date_joined')}),
    )
    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('phone_number', 'password1', 'password2'),
        }),
    )
    search_fields = ('phone_number', 'full_name', 'nick_name', 'email')
    ordering = ('phone_number',)


admin.site.register(User, CustomUserAdmin)



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: models.py
# Project: django-panel-core
# File Created: Tuesday, 25th February 2020 1:26:07 am
# 
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
# 
# Last Modified: Tuesday, 25th February 2020 1:26:09 am
# Modified By: Arif Dzikrullah (ardzix@hotmail.com>)
# 
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-panel-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


import uuid
from django.db import models
from django.conf import settings
from django.db.models.signals import pre_save, post_save
from django.dispatch import receiver
from django.utils import timezone
from django.core.mail import send_mail
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.models import PermissionsMixin
from django.contrib.auth.base_user import AbstractBaseUser, BaseUserManager
from ...libs.base62 import base62_encode


class UserManager(BaseUserManager):
    use_in_migrations = True

    def _create_user(self, phone_number, email, password, **extra_fields):
        """
        Create and save a user with the given username, email, and password.
        """
        if not phone_number:
            raise ValueError('The given username must be set')
        email = self.normalize_email(email)
        # username = self.model.normalize_username(username)
        user = self.model(
            phone_number=phone_number,
            email=email,
            **extra_fields)
        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_user(self, phone_number, email=None,
                    password=None, **extra_fields):
        extra_fields.setdefault('is_staff', False)
        extra_fields.setdefault('is_superuser', False)
        return self._create_user(phone_number, email, password, **extra_fields)

    def create_superuser(self, phone_number, email, password, **extra_fields):
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)

        if extra_fields.get('is_staff') is not True:
            raise ValueError('Superuser must have is_staff=True.')
        if extra_fields.get('is_superuser') is not True:
            raise ValueError('Superuser must have is_superuser=True.')

        return self._create_user(phone_number, email, password, **extra_fields)


class AbstractUser(AbstractBaseUser, PermissionsMixin):
    id62 = models.CharField(
        max_length=100,
        db_index=True,
        blank=True,
        null=True)
    phone_number = models.CharField(unique=True, max_length=20,
                                    help_text=_(
                                        'Required. 20 characters or fewer. digits only.'),
                                    )

    full_name = models.CharField(_('full name'), max_length=150, blank=True)
    nick_name = models.CharField(_('nick name'), max_length=150, blank=True)
    email = models.EmailField(_('email address'), unique=True)
    is_staff = models.BooleanField(
        _('staff status'),
        default=False,
        help_text=_(
            'Designates whether the user can log into this admin site.'),
    )
    is_active = models.BooleanField(
        _('active'),
        default=False,
        help_text=_(
            'Designates whether this user should be treated as active. '
            'Unselect this instead of deleting accounts.'
        ),
    )
    is_sent_email = models.BooleanField(default=True)
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    objects = UserManager()

    EMAIL_FIELD = 'email'
    USERNAME_FIELD = 'phone_number'
    REQUIRED_FIELDS = ['email']

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        abstract = True

    def clean(self):
        super().clean()
        self.email = self.__class__.objects.normalize_email(self.email)

    def get_full_name(self):
        """
        Return the first_name plus the last_name, with a space in between.
        """
        return self.full_name

    def get_short_name(self):
        """Return the short name for the user."""
        return self.nick_name

    def email_user(self, subject, message, from_email=None, **kwargs):
        """Send an email to this user."""
        send_mail(subject, message, from_email, [self.email], **kwargs)

    def save(self, *args, **kwargs):
        instance = super(AbstractUser, self).save(*args, **kwargs)
        if self.id and not self.id62:
            self.id62 = base62_encode(self.id)
            kwargs['force_insert'] = False
            instance = super(AbstractUser, self).save(*args, **kwargs)
        return instance


class User(AbstractUser):
    """
    Users within the Django authentication system are represented by this
    model.

    Username and password are required. Other fields are optional.
    """

    def __str__(self):
        return '%s <%s>' % (self.full_name, self.email)

    def get_groups(self):
        return ', '.join(list(self.groups.values_list('name', flat=True)))

    class Meta(AbstractUser.Meta):
        swappable = 'AUTH_USER_MODEL'
        unique_together = ('phone_number', 'full_name')


class RegisterToken(models.Model):
    phone_number = models.CharField(unique=True, max_length=20,
                                    help_text=_(
                                        'Required. 20 characters or fewer. digits only.'),
                                    )
    token = models.CharField(_('token'), max_length=150, blank=True)

    class Meta:
        verbose_name = _('register token')
        verbose_name_plural = _('register tokens')


class EmailVerification(models.Model):
    email = models.EmailField()
    code = models.CharField(max_length=100)
    is_verified = models.BooleanField(default=False)
    user = models.ForeignKey(
        User,
        on_delete=models.CASCADE,
        blank=True,
        null=True)

    def __str__(self):
        return self.email

    class Meta:
        verbose_name = _('Email Varification')
        verbose_name_plural = _('Email Varifications')


def send_verification_email(email, user, base_url=None, *args, **kwargs):
    from ...libs.email import send_mail
    from ...libs.otp import generate_otp_code
    from django.conf import settings

    subject_template_name = "email/email_verify.txt"
    html_email_template_name = "email/email_verify.html"
    email_template_name = html_email_template_name
    code = generate_otp_code(6)

    context = {
        "code": code,
        "name": user.full_name
    }

    send_mail(
        subject_template_name,
        email_template_name,
        html_email_template_name,
        context,
        email, cc=getattr(settings, "MAIL_NOTIFICATION_CC", [])
    )

    ev, created = EmailVerification.objects.get_or_create(
        email=email
    )
    ev.code = code
    ev.is_verified = False
    ev.save()

    return ev


@receiver(pre_save, sender=User)
def verify_email(sender, instance, **kwargs):
    from django.conf import settings

    if getattr(settings, 'AUTO_VERIFY_EMAIL', False):
        email = instance.email
        if instance.is_sent_email:
            existed_user = User.objects.filter(id=instance.id).first()
            if not existed_user:
                send_verification_email(
                    email,
                    instance,
                    is_reset_password=True
                )
            else:
                if email != existed_user.email:
                    send_verification_email(email, instance)


@receiver(post_save, sender=User)
def save_ev(sender, instance, **kwargs):
    if getattr(settings, 'AUTO_VERIFY_EMAIL', False):
        ev = EmailVerification.objects.filter(email=instance.email).last()
        if ev:
            ev.user = instance
            ev.save()



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: moment.py
# Project: django-enterprise-core
# File Created: Thursday, 16th August 2018 11:49:22 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Thursday, 16th August 2018 11:49:22 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-enterprise-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from datetime import datetime, timedelta
from django.utils.dateformat import format


def to_timestamp(dt):
    return format(dt, 'U')


def get_today_epoch():
    now = datetime.utcnow()
    return now.replace(now.year, now.month, now.day, 0, 0, 0, 0)


def get_difference_epoch(epoch):
    diff = datetime.utcnow() - epoch
    return int(diff.total_seconds() * 1000000)


def get_next_monday(today):
    today = today.date()
    return today + timedelta(days=-today.weekday(), weeks=1)


def get_last_monday(today, ):
    today = today.date()
    return today - timedelta(days=today.weekday(), )



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: midtrans.py
# Project: core.wecare.id
# File Created: Monday, 7th January 2019 10:15:09 pm
# 
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
# 
# Last Modified: Monday, 7th January 2019 10:15:09 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
# 
# Handcrafted and Made with Love
# Copyright - 2018 Wecare.Id, wecare.id
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


import requests
import base64
import json
import ssl
import datetime

from django.urls.base import NoReverseMatch
from django.shortcuts import reverse
from django.utils.translation import gettext_lazy as _
from django.conf import settings
from django.core.serializers.json import DjangoJSONEncoder

PAYMENT_TYPE_CHOICES = (
    ('gopay', _('GoPay')),
    ('telkomsel_cash', _('Telkomsel Cash')),
    ('indosat_dompetku', _('Indosat Dompetku')),
    ('mandiri_ecash', _('LINE Pay e-cash|mandiri e-cash')),
    ('bca_klikpay', _('BCA Klickpay')),
    ('bca_klikbca', _('Klik BCA')),
    ('mandiri_clickpay', _('Mandiri Klikpay')),
    ('bri_epay', _('Epay BRI')),
    ('cimb_clicks', _('CIMB Clicks')),
    ('bank_transfer', _('Bank Transfer')),
    ('credit_card', _('Credit Card')),
)

PAYMENT_STATUSES = (
    ('authorize', _('Transaction authorizes the credit card.')),
    ('capture', _('Transaction captures the credit card balance. Will be settled as batch.')),
    ('settlement', _('Transaction is settled.')),
    ('deny', _('Payment provider rejects the credentials used for payment.')),
    ('pending', _('Transaction is made available in Midtrans to be paid.')),
    ('cancel', _('Transaction is cancelled.')),
    ('refund', _('Transaction is marked to be refunded.')),
    ('partial_refund', _('Transaction is marked to be partially refunded.')),
    ('chargeback', _('Transaction is marked to be charged back.')),
    ('partial_chargeback', _('Transaction is marked to be partially charged back.')),
    ('expire', _('Transaction no longer available to be paid or processed')),
    ('failure', _('Unexpected error during transaction processing')),

    ('preparing', _('Preparing data for transaction to midtrans'))
)


class _BaseMidtransPay(object):
    '''
    Abstract class for payment with midtrans
    '''
    payload = {}
    invoice = None

    def __init__(self, payment_type, invoice):
        self.invoice = invoice
        self.payload['payment_type'] = payment_type

        if invoice:
            user = invoice.created_by
            self.add_customer_details({
                'first_name': user.full_name,
                'email': user.email,
                'phone': user.phone_number,
            })
            try:
                self.add_callbacks(
                    {
                        'finish': '%s?inv=%s' % (
                            reverse('donation:success'), 
                            invoice.number
                        ),
                        'pending': '%s?inv=%s' % (
                            reverse('payment:status'), 
                            invoice.number
                        ),
                    }
                )
            except NoReverseMatch as e:
                print(e)

            items = []
            for item in invoice.get_items():
                items.append({
                    "id": item.id62,
                    "price": int(item.amount),
                    "quantity": item.qty,
                    "name": item.item_name,
                })
            self.add_item_details(items)

    def add_payload(self, *args, **kwargs):
        self.payload = {**self.payload, **kwargs}

    def add_customer_details(self, data):
        self.add_payload(customer_details = data)
    
    def add_callbacks(self, data):
        self.add_payload(callbacks = data)
    
    def add_item_details(self, data):
        self.add_payload(item_details = data)

    def set_transaction_details(self, order_id, gross_amount):
        if getattr(settings, 'PRODUCTION', False) is False:
            order_id = '%s-%s-%s' %(
                order_id.split('-')[0],
                datetime.datetime.timestamp(datetime.datetime.now()),
                order_id.split('-')[1]
            )

        transaction_details = {
            'order_id' : order_id,
            'gross_amount' : gross_amount
        }
        self.add_payload(transaction_details=transaction_details)

    def get_payload(self):
        return self.payload

    def request(self, url):
        basic_auth = '%s:' % getattr(settings, 'API_SERVER_KEY')
        encoded_basic_auth = base64.b64encode(basic_auth.encode()).decode('utf-8')
        headers = {
            'Content-Type' : 'application/json',
            'Accept' : 'application/json',
            'Authorization' : 'Basic %s' % encoded_basic_auth
        }
        print(self.payload)
        r = requests.post(url, data=json.dumps(self.payload, cls=DjangoJSONEncoder), headers=headers)
        r_dict = r.json()

        return r_dict

    def charge(self, user, amount):
        from enterprise.structures.transaction.models.midtrans import Midtrans

        if not self.invoice:
            raise Exception('Invoice is null')

        midtrans = Midtrans(
            payment_type = self.payload['payment_type'],
            transaction_status = 'preparing',
            amount = amount,
            payload = self.payload
        )
        midtrans.created_by = user
        midtrans.save()

        self.set_transaction_details(order_id=self.invoice.number+'-'+midtrans.id62, gross_amount=int(amount))

        result = self.request(self.get_charge_url())
        if 'transaction_id' in result:
            midtrans.transaction_id = result['transaction_id']
        if 'transaction_status' in result:
            midtrans.transaction_status = result['transaction_status']
        midtrans.responses = result
        midtrans.save()
        return result

    def get_charge_url(self):
        return '%s/v2/charge' % getattr(settings, 'API_URL',
                                        'https://api.sandbox.midtrans.com')


class Gopay(_BaseMidtransPay):
    def __init__(self, invoice, *args, **kwargs):
        callback_url = kwargs.get('callback_url')
        super(Gopay, self).__init__('gopay', invoice=invoice)
        self.add_payload(
            gopay = {
                'enable_callback' : True if callback_url else False,
                'callback_url' : callback_url
            }
        )


class BankTransfer(_BaseMidtransPay):
    def __init__(self, invoice, *args, **kwargs):
        super(BankTransfer, self).__init__('bank_transfer', invoice=invoice)
        self.add_payload(
            bank_transfer = {
                'bank' : kwargs.get('bank'),
            }
        )


class MandiriEChannel(_BaseMidtransPay):
    def __init__(self, invoice, *args, **kwargs):
        super(MandiriEChannel, self).__init__('echannel', invoice=invoice)
        self.add_payload(
            echannel = {
                'bill_info1' : 'Payment for invoice %s' % invoice.number,
                'bill_info2' : 'debt',
            }
        )


class CreditCard(_BaseMidtransPay):
    def __init__(self, invoice, *args, **kwargs):
        super(CreditCard, self).__init__('credit_card', invoice=invoice)
        self.add_payload(
            credit_card = {
                'card': kwargs.get('card'),
                'dynamic_descriptor': {
                    'merchant_name': 'Wecare.Id',
                    'city_name': 'Jakarta',
                    'country_code': 'ID'
                },
                'authentication': True,
                'callback_type': 'js_event'
            }
  
        )

class Snap(_BaseMidtransPay):
    def __init__(self, invoice, *args, **kwargs):
        super(Snap, self).__init__('snap', invoice=invoice)
        payment_channel = kwargs.get('payment_channel')
        if not payment_channel:
            payment_channel = PAYMENT_TYPE_CHOICES
        self.add_payload(credit_card = {'secure':True})
        self.add_payload(enabled_payments = list(k for k,v in payment_channel))

    def get_charge_url(self):
        return '%s/snap/v1/transactions' % getattr(settings, 'APP_URL',
                                        'https://app.sandbox.midtrans.com')


'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: brand.py
# Project: django-enterprise-core
# File Created: Tuesday, 21st August 2018 11:51:25 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Friday, 26th October 2018 3:45:31 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-enterprise-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''
from django.db.models import Q


class BrandManager(object):
    instance = None
    brands = []

    def __init__(self, *args, **kwargs):
        if kwargs.get('instance'):
            self.set_by_instance(kwargs.get('instance'))
        if kwargs.get('owner'):
            self.set_by_owner(kwargs.get('owner'))
        if kwargs.get('executive'):
            self.set_by_executive(kwargs.get('executive'))
        if kwargs.get('staff'):
            self.set_by_staff(kwargs.get('staff'))
        if kwargs.get('user'):
            self.set_by_user(kwargs.get('user'))

    def set_by_instance(self, instance):
        self.brands.append(instance)
        self.change_instance()

    def set_by_owner(self, owner):
        self.brands = self.brand_objects().filter(owned_by=owner).all()
        self.change_instance()

    def set_by_executive(self, executive):
        self.brands = self.brand_objects().filter(executive=executive).all()
        self.change_instance()

    def set_by_staff(self, staff):
        self.brands = self.brand_objects().filter(staffs=staff).all()
        self.change_instance()

    def set_by_user(self, user):
        self.brands = self.brand_objects().filter(
            Q(owned_by=user) | Q(executive=user) | Q(staffs=user)
        ).all()
        self.change_instance()

    def change_instance(self, index=0, request=None):
        if len(self.brands) <= index:
            return False

        self.instance = self.brands[index]
        if request:
            request.session['brand'] = self.instance
            return request

        return True

    def get_brands(self):
        return self.brands

    def get_staffs(self):
        if not self.instance:
            raise Exception("No instance specified")

        return self.instance.staffs.all()

    def get_owner(self):
        if not self.instance:
            raise Exception("No instance specified")

        return self.instance.owned_by

    def get_executive(self):
        if not self.instance:
            raise Exception("No instance specified")

        return self.instance.executive

    def get_company(self):
        if not self.instance:
            raise Exception("No instance specified")

        return self.instance.company

    def get_company_staffs(self):
        if not self.instance:
            raise Exception("No instance specified")

        return self.get_company().staffs.all() if self.get_company() else []

    def get_company_owner(self):
        if not self.instance:
            raise Exception("No instance specified")

        return self.get_company().owned_by if self.get_company() else None

    def is_staff(self, user):
        return True if user in self.get_staffs() else False

    def is_executive(self, user):
        return True if user == self.get_executive() else False

    def is_owner(self, user):
        return True if user == self.get_owner() else False

    def is_company_staff(self, user):
        return True if user in self.get_company_staffs() else False

    def is_company_owner(self, user):
        return True if user == self.get_company_owner() else False

    def get_permissions(self, user):
        if not self.instance:
            raise Exception("No instance specified")

        permissions = []
        if self.is_staff(user):
            permissions.append('staff')
        if self.is_executive(user):
            permissions.append('executive')
        if self.is_owner(user):
            permissions.append('owner')
        return permissions

    def get_company_permissions(self, user):
        if not self.instance:
            raise Exception("No instance specified")

        permissions = []
        if self.is_company_staff(user):
            permissions.append('staff')
        if self.is_company_owner(user):
            permissions.append('owner')
        return permissions



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: forms.py
# Project: django-enterprise-core
# File Created: Wednesday, 22nd August 2018 12:50:12 am
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Wednesday, 22nd August 2018 12:50:13 am
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-enterprise-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''

from django.forms import *
from .models import *
from django.contrib.auth.models import Group, Permission
from django.contrib.auth import get_user_model as U
from django.contrib.contenttypes.models import ContentType


class UserForm(ModelForm):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['password'].required = False

    class Meta:
        model = U()
        fields = (
            'full_name',
            'email',
            'phone_number',
            'groups',
            'password',
            'is_active',
            'is_staff',
            'is_superuser',
        )


class GroupForm(ModelForm):
    class Meta:
        model = Group
        fields = {
            'name': models.CharField(max_length=255),
            'permissions': ModelMultipleChoiceField(
                queryset=Permission.objects.all(),
            )
        }


class PermissionForm(ModelForm):
    class Meta:
        model = Permission
        fields = {
            'name': models.CharField(max_length=255),
            'codename': models.CharField(max_length=255),
            'content_type': ModelChoiceField(
                queryset=ContentType.objects.all(),
            )
        }



from ...structures.common.views import apilogger


class APILogMiddleware(object):
    """
    Middleware for Log API every call
    will be add in settings.py MIDDLEWARE
    """

    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.

        response = self.get_response(request)

        if (request.META.get("HTTP_HOST") == 'api.helloyuna.io' or
                request.META.get("HTTP_HOST") == '45.118.134.111:10001'):

            api_meta = {"user": None if request.user.is_anonymous else request.user,
                        "app_id": request.META.get('HTTP_X_YUNA_APP_ID'),
                        "uid": request.META.get('HTTP_X_YUNA_SECRET'),
                        "latitude": request.GET.get("lat"),
                        "longitude": request.GET.get("lng"),
                        "endpoint": request.META.get("PATH_INFO"),
                        "request_method": request.method
                        }

            apilogger(api_meta)

        return response


class PAYLogMiddleware(object):
    """
    Middleware for Log API every call
    will be add in settings.py MIDDLEWARE
    """

    def __init__(self, get_response):
        self.get_response = get_response
        # One-time configuration and initialization.

    def __call__(self, request):
        # Code to be executed for each request before
        # the view (and later middleware) are called.

        response = self.get_response(request)

        if (request.META.get("HTTP_HOST") == 'pay.helloyuna.io' or
                request.META.get("HTTP_HOST") == '45.118.134.111:8401'):

            api_meta = {"user": None if request.user.is_anonymous else request.user,
                        "app_id": request.META.get('HTTP_X_YUNA_APP_ID'),
                        "uid": request.META.get('HTTP_X_YUNA_SECRET'),
                        "latitude": request.GET.get("lat"),
                        "longitude": request.GET.get("lng"),
                        "endpoint": request.META.get("PATH_INFO"),
                        "request_method": request.method
                        }

            apilogger(api_meta)

        return response



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: email.py
# Project: core.wecare.id
# File Created: Wednesday, 20th February 2019 3:30:08 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Wednesday, 20th February 2019 3:30:08 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Handcrafted and Made with Love
# Copyright - 2018 Wecare.Id, wecare.id
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


import json
from django.core.mail import EmailMultiAlternatives
from django.template import loader
from django.conf import settings


def send_mail(subject_template_name, email_template_name, html_email_template_name,
              context, to_email, from_email=None, mandrill_template=None, mandrill_variables=None,
              cc=[]):
    """
    Sends a django.core.mail.EmailMultiAlternatives to `to_email`.
    """
    if not from_email:
        from_email = getattr(settings, 'FROM_EMAIL')

    subject = loader.render_to_string(subject_template_name, context)

    # Email subject *must not* contain newlines
    subject = ''.join(subject.splitlines())
    body = loader.render_to_string(email_template_name, context)

    headers = {}

    if mandrill_template:
        headers["X-MC-Template"] = mandrill_template

    if mandrill_variables:
        headers["X-MC-MergeVars"] = json.dumps(mandrill_variables)

    email_message = EmailMultiAlternatives(
        subject, body, from_email, [to_email], headers=headers, cc=cc)

    if html_email_template_name is not None:
        html_email = loader.render_to_string(html_email_template_name, context)
        email_message.attach_alternative(html_email, 'text/html')

    try:
        email_message.send()
    except Exception as e:
        if settings.DEBUG:
            print (str(e))
        else:
            raise e


from rest_framework.permissions import IsAuthenticated, BasePermission

SAFE_METHODS = ['GET', ]


class IsOwnerAuthenticated(IsAuthenticated):
    """
    Allows access only to authenticated users.
    """

    def has_permission(self, request, view):
        is_authenticated = super(
            IsOwnerAuthenticated,
            self).has_permission(
            request,
            view)
        return is_authenticated and view.get_object().owned_by == request.user


class IsCreatorAuthenticated(IsAuthenticated):
    """
    Allows access only to authenticated users.
    """

    def has_permission(self, request, view):
        is_authenticated = super(
            IsCreatorAuthenticated,
            self).has_permission(
            request,
            view)
        return is_authenticated and view.get_object().created_by == request.user


class IsOwnerOrReadOnly(BasePermission):
    """
    The request is authenticated as a user, or is a read-only request.
    """

    def has_permission(self, request, view):
        if request.user and request.user.is_authenticated:
            return True
        elif view.action in SAFE_METHODS:
            return True
        return False

    def has_object_permission(self, request, view, obj):

        if view.action in ['list', 'retrieve']:
            return True

        # if not list or retrieve, if not log in then always False
        if not request.user.is_authenticated:
            return False
        return obj.owned_by == request.user


class IsReadOnly(BasePermission):
    """
    The request is authenticated as a user, or is a read-only request.
    """

    def has_permission(self, request, view):

        return (
            request.method in SAFE_METHODS
        )

    def has_object_permission(self, request, view, obj):
        if view.action in ['list', 'retrieve']:
            return True

        # if not list or retrieve, if not log in then always False
        return False



from django.contrib import admin
from .models import Tracker


class TrackerAdmin(admin.ModelAdmin):
    list_display = ('created_at', 'created_by', 'ip', 'trigger_action')


admin.site.register(Tracker, TrackerAdmin)



from django import forms
from django.contrib.auth import (
    get_user_model, password_validation,
)
from django.contrib.auth.models import User
from django.utils.translation import gettext, gettext_lazy as _
from django.contrib.auth.forms import UsernameField, ReadOnlyPasswordHashField
from .models import User


class CustomUserCreationForm(forms.ModelForm):
    """
    A form that creates a user, with no privileges, from the given username and
    password.
    """
    error_messages = {
        'password_mismatch': _("The two password fields didn't match."),
    }
    password1 = forms.CharField(
        label=_("Password"),
        strip=False,
        widget=forms.PasswordInput,
        help_text=password_validation.password_validators_help_text_html(),
    )
    password2 = forms.CharField(
        label=_("Password confirmation"),
        widget=forms.PasswordInput,
        strip=False,
        help_text=_("Enter the same password as before, for verification."),
    )

    class Meta:
        model = User
        fields = ("phone_number",)
        field_classes = {'phone_number': UsernameField}

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self._meta.model.USERNAME_FIELD in self.fields:
            self.fields[self._meta.model.USERNAME_FIELD].widget.attrs.update({
                                                                             'autofocus': True})

    def clean_password2(self):
        password1 = self.cleaned_data.get("password1")
        password2 = self.cleaned_data.get("password2")
        if password1 and password2 and password1 != password2:
            raise forms.ValidationError(
                self.error_messages['password_mismatch'],
                code='password_mismatch',
            )
        return password2

    def _post_clean(self):
        super()._post_clean()
        # Validate the password after self.instance is updated with form data
        # by super().
        password = self.cleaned_data.get('password2')
        if password:
            try:
                password_validation.validate_password(password, self.instance)
            except forms.ValidationError as error:
                self.add_error('password2', error)

    def save(self, commit=True):
        user = super().save(commit=False)
        user.set_password(self.cleaned_data["password1"])
        if commit:
            user.save()
        return user


class CustomUserChangeForm(forms.ModelForm):
    password = ReadOnlyPasswordHashField(
        label=_("Password"),
        help_text=_(
            "Raw passwords are not stored, so there is no way to see this "
            "user's password, but you can change the password using "
            "<a href=\"{}\">this form</a>."
        ),
    )

    class Meta:
        model = User
        fields = '__all__'
        field_classes = {'phone_number': UsernameField}

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        password = self.fields.get('password')
        if password:
            password.help_text = password.help_text.format('../password/')
        user_permissions = self.fields.get('user_permissions')
        if user_permissions:
            user_permissions.queryset = user_permissions.queryset.select_related(
                'content_type')

    def clean_password(self):
        # Regardless of what the user provides, return the initial value.
        # This is done here, rather than on the field, because the
        # field does not have access to the initial value
        return self.initial["password"]



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: forms.py
# Project: <<projectname>>
# File Created: Thursday, 21st June 2018 4:49:59 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         http://ardz.xyz>
#
# Last Modified: Thursday, 21st June 2018 4:49:59 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Crafted by Pro
# Copyright - <<year>> Ardz & Co, -
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''
from django.forms import *
from models import *
from django.contrib.contenttypes.models import ContentType


class LogForm(ModelForm):
    note = CharField(
        max_length=255,
        required=False
    )
    content_id62 = CharField(
        max_length=10,
        required=False,
        widget=TextInput(attrs={'class': 'form-control'})
    )

    class Meta:
        model = Log
        exclude = ("content_model", "content_object", "object_id")
        widgets = {
            'content_type': Select(attrs={'class': 'full-width', 'data-placeholder': 'Content Type', 'data-init-plugin': 'select2'}),
            'mentions': SelectMultiple(attrs={'class': 'full-width', 'data-init-plugin': 'select2'}),
            'logged_by': Select(attrs={'class': 'full-width', 'data-init-plugin': 'select2'}),
        }
        labels = {
            'object_id': "Object",
        }



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: serializer.py
# Project: core.lakon.app
# File Created: Friday, 14th September 2018 6:13:56 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Friday, 14th September 2018 6:14:13 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Lakon, lakon.app
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''
import timeago
from datetime import timedelta
from rest_framework.serializers import ModelSerializer, ValidationError, SerializerMethodField
from ..nonce import NonceObject


class CommonSerializer(object):
    created_at = SerializerMethodField()
    published_at = SerializerMethodField()
    deleted_at = SerializerMethodField()
    point = SerializerMethodField()

    def get_created_at(self, obj):
        if obj.created_at:
            return {
                'utc': obj.created_at,
                'timestamp': obj.created_at_timestamp,
                'timeago': timeago.format(obj.created_at.replace(tzinfo=None) + timedelta(hours=7))
            }
        return None

    def get_published_at(self, obj):
        if obj.published_at:
            return {
                'utc': obj.published_at,
                'timestamp': obj.published_at_timestamp,
                'timeago': timeago.format(obj.published_at.replace(tzinfo=None) + timedelta(hours=7))
            }
        return None

    def get_deleted_at(self, obj):
        if obj.deleted_at:
            return {
                'utc': obj.deleted_at,
                'timestamp': obj.deleted_at_timestamp,
                'timeago': timeago.format(obj.deleted_at.replace(tzinfo=None) + timedelta(hours=7))
            }
        return None

    def get_point(self, obj):
        point = obj.point
        result = {'latitude': 0, 'longitude': 0}
        try:
            result = {
                'latitude': point.y,
                'longitude': point.x
            }
        except BaseException:
            pass

        return result


class LakonModelSerializer(ModelSerializer):
    def create(self, validated_data):
        if 'nonce' not in validated_data:
            raise ValidationError({'detail': 'Please provide nonce'})

        nonce = validated_data['nonce']
        model = self.Meta.model
        no = NonceObject(model=model, nonce=nonce)

        validated_data['created_by'] = self.context.get('request').user

        if no.is_exist():
            return super(LakonModelSerializer, self).update(
                no.get_instance(), validated_data)
        else:
            return super(LakonModelSerializer, self).create(validated_data)

    def update(self, instance, validated_data):
        if 'nonce' not in validated_data:
            raise ValidationError({'detail': 'Please provide nonce'})

        nonce = validated_data['nonce']
        model = self.Meta.model
        no = NonceObject(model=model, nonce=nonce)

        if not no.get_instance() == self.instance:
            raise ValidationError({'detail': 'Nonce is not matched'})

        return super(LakonModelSerializer, self).update(
            self.instance, validated_data)



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: form.py
# Project: django-panel-core
# File Created: Tuesday, 17th September 2019 6:31:54 pm
# 
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
# 
# Last Modified: Tuesday, 17th September 2019 6:31:55 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
# 
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-panel-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''

from django.views.generic import TemplateView 
from django.shortcuts import get_object_or_404, redirect
from django.contrib import messages

class FormViewMixin(TemplateView):
    template_name = None
    model_class = None
    form_class = None

    def get(self, request):
        edit = request.GET.get("edit")
        instance = None
        if edit:
            instance = get_object_or_404(
                self.model_class, id62=edit)
            form = self.form_class(
                instance=instance,
            )
        else:
            form = self.form_class()

        return self.render_to_response({
            'title': self.model_class.__name__,
            'form': form,
            'form_title': self.model_class.__name__
        })

    def post(self, request):
        edit = request.GET.get("edit")
        instance = None
        if edit:
            instance = get_object_or_404(
                self.model_class, id62=edit)
            form = self.form_class(
                request.POST,
                request.FILES,
                instance=instance
            )
        else:
            form = self.form_class(request.POST, request.FILES)

        if form.is_valid():
            obj = form.save(commit=False)
            if not edit:
                obj.created_by = request.user
            else:
                obj.updated_by = request.user
            obj.save()
            form.save_m2m()
            messages.success(
                request,
                '%s has been saved.' %
                self.model_class.__name__)
            return redirect(request.META.get('HTTP_REFERER'))

        return self.render_to_response({
            'title': self.model_class.__name__,
            'form': form,
            'form_title': self.model_class.__name__
        })


'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: form.py
# Project: django-enterprise-core
# File Created: Tuesday, 21st August 2018 11:56:49 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Tuesday, 21st August 2018 11:56:49 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-enterprise-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.forms.utils import ErrorList
from django.contrib.auth.forms import *
from django import forms
from django.conf import settings
from django.utils.translation import gettext_lazy as _
from crispy_forms import helper, layout, bootstrap

from enterprise.structures.authentication.models import User, EmailVerification, send_verification_email


class ErrorDiv(ErrorList):
    def __str__(self):
        return self.as_divs()

    def __unicode__(self):
        return self.as_divs()

    def as_divs(self):
        if not self:
            return ''
        else:
            errors = ''.join(['<div class="error">%s</div>' % e for e in self])
            return '<div class="errors">%s</div>' % errors


class AuthForm(forms.Form):
    phone_number = forms.CharField(
        label=False,
        widget=forms.TextInput(attrs={'placeholder': _('Email/Phone number')})
    )
    password = forms.CharField(
        label=False,
        widget=forms.PasswordInput(attrs={'placeholder': _('Password')})
    )

    def clean(self):
        cleaned_data = super().clean()
        phone_number = cleaned_data['phone_number']
        password = cleaned_data['password']
        user = authenticate(phone_number=phone_number, password=password)
        if not user:
            if getattr(settings, 'AUTO_VERIFY_EMAIL'):
                user_temp = User.objects.filter(email=phone_number).first()
                if user_temp and user_temp.is_active:
                    ev = EmailVerification.objects.filter(
                        email=phone_number).first()
                    if ev:
                        if ev.is_verified:
                            user = authenticate(
                                phone_number=user_temp.phone_number, password=password)
                        else:
                            self.add_error(
                                'phone_number',
                                _('Your email has not been verified, please check your email to verify it'))
                            self.verify_email(phone_number, user_temp)
            else:
                user = authenticate(
                            email=phone_number, password=password)

        if user:
            if user.is_active:
                cleaned_data['user'] = user
            else:
                self.add_error('phone_number', _('Your account is not active'))
        else:
            self.add_error(
                'phone_number',
                _('Email/Phone and Password missmatch'))

        return cleaned_data

    def verify_email(self, email, user):
        ev = send_verification_email(email, user, is_reset_password=True)
        ev.user = user
        ev.save()

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.helper = helper.FormHelper()
        self.helper.form_tag = False
        self.helper.layout = layout.Layout(
            bootstrap.PrependedText(
                'phone_number',
                '<i class="fas fa-user"></i>',
                wrapper_class='col-md-12 m-b-10'
            ),
            bootstrap.PrependedText(
                'password',
                '<i class="fas fa-lock"></i>',
                wrapper_class='col-md-12'
            ),
        )
        self.helper.render_unmentioned_fields = True
        self.helper.disable_csrf = True


class ChangePasswordForm(PasswordChangeForm):
    def __init__(self, *args, **kwargs):
        super(ChangePasswordForm, self).__init__(*args, **kwargs)

        self.error_class = ErrorDiv

        self.fields['old_password'].widget.attrs = {
            'class': 'form-control'
        }
        self.fields['new_password1'].widget.attrs = {
            'class': 'form-control'
        }
        self.fields['new_password2'].widget.attrs = {
            'class': 'form-control'
        }


class SetPasswordForm(SetPasswordForm):
    def __init__(self, *args, **kwargs):
        super(SetPasswordForm, self).__init__(*args, **kwargs)

        self.error_class = ErrorDiv

        self.fields['new_password1'].widget.attrs = {
            'class': 'form-control'
        }
        self.fields['new_password2'].widget.attrs = {
            'class': 'form-control'
        }


class RegisterForm(forms.Form):
    full_name = forms.CharField(widget=forms.TextInput(
        attrs={'placeholder': 'Full Name', 'class': 'form-control'}
    ))
    phone_number = forms.CharField(widget=forms.TextInput(
        attrs={'placeholder': 'Phone Number', 'class': 'form-control'}
    ))



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: __init__.py
# Project: core.ayopeduli.id
# File Created: Thursday, 1st November 2018 1:47:22 am
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Thursday, 1st November 2018 1:47:22 am
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Peduli sesama, sejahtera bersama
# Copyright - 2018 Ayopeduli.Id, ayopeduli.id
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: permission.py
# Project: django-enterprise-core
# File Created: Wednesday, 22nd August 2018 1:48:22 am
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Wednesday, 22nd August 2018 1:48:22 am
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-enterprise-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.views.generic import TemplateView
from django.shortcuts import redirect
from django.contrib import messages
from django.contrib.auth.models import Permission
from enterprise.libs.view import ProtectedMixin
from datatable import Datatable
from ..forms import PermissionForm


class PermissionView(ProtectedMixin, TemplateView):
    template_name = "superuser/permission.html"

    def get(self, request, *args, **kwargs):
        if request.GET.get('draw', None) is not None:
            return self.datatable(request)

        return self.render_to_response({
            'title': "Tracker enterprise - %s" % Permission.__name__,
        })

    def delete(self, request):
        o_id = request.body.decode('utf-8').split("=")[1]
        qs = Permission.objects.filter(id__exact=o_id).first()
        qs.delete()
        return self.render_to_response({})

    def datatable(self, request):
        qs = Permission.objects.all()
        defer = ['id', 'name', 'codename', 'content_type']

        d = Datatable(request, qs, defer, key="id")
        return d.get_data()


class PermissionFormView(ProtectedMixin, TemplateView):
    template_name = "superuser/permission.form.html"

    def get(self, request, *args, **kwargs):
        edit = request.GET.get("edit")

        if edit:
            form = PermissionForm(instance=Permission.objects.get(id=edit))
        else:
            form = PermissionForm()

        return self.render_to_response({"form": form})

    def post(self, request):
        edit = request.GET.get("edit")

        if edit:
            form = PermissionForm(
                request.POST,
                instance=Permission.objects.get(
                    id=edit))
        else:
            form = PermissionForm(request.POST)

        if form.is_valid():
            permission = form.save(commit=False)
            permission.save()
            messages.success(
                request, _(
                    'Permission (%s) has been saved.' %
                    permission.name))
            return redirect("superuser:permission")
        else:
            return self.render_to_response({"form": form})



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: otp.py
# Project: django-panel-core
# File Created: Tuesday, 25th February 2020 12:57:24 am
# 
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
# 
# Last Modified: Tuesday, 25th February 2020 12:57:24 am
# Modified By: Arif Dzikrullah (ardzix@hotmail.com>)
# 
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-panel-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''
import random



def generate_otp_code(length):
    """Generate OTP code with custom length

    Arguments:
        length {integer} -- Length of code [Max 6]
    """

    otp = ""
    for i in range(length):
        otp += str(random.randint(1, 9))

    return otp


'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: mixin.py
# Project: <<projectname>>
# File Created: Wednesday, 6th November 2019 2:17:25 am
# 
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         http://ardz.xyz>
# 
# Last Modified: Wednesday, 6th November 2019 2:17:25 am
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
# 
# Crafted by Pro
# Copyright - <<year>> Ardz & Co, -
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


import jwt
from rest_framework.permissions import *
from django.conf import settings

class JWTAuthenticated(BasePermission):
    def has_permission(self, request, view):
        if not settings.VALIDATE_JWT:
            return True
        if not request.data.get('encoded'):
            raise exceptions.ParseError(detail='Please wrap your data with JWT in \'encoded\' ')
        try:
            decoded = jwt.decode(request.data.get('encoded'), settings.JWT_SECRET, algorithms=settings.JWT_ALGORITHMS)
        except Exception as e:
            raise exceptions.AuthenticationFailed(detail=str(e))
        request._full_data = decoded
        # decoded = jwt.decode(encoded, settings.JWT_SECRET, algorithms=settings.JWT_ALGORITHMS)
        return True


'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: pay_constants.py
# Project: core.wecare.id
# File Created: Wednesday, 5th December 2018 12:07:34 am
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Wednesday, 5th December 2018 12:07:34 am
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Handcrafted and Made with Love
# Copyright - 2018 Wecare.Id, wecare.id
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from decimal import Decimal
from django.utils.translation import gettext_lazy as _

BILL_STATUSES = (
    ('authorize', _('Authorized')),
    ('capture', _('Captured')),
    ('reviewed', _('Reviewed')),
    ('settlement', _('Settled')),
    ('deny', _('Denied')),
    ('pending', _('Pending')),
    ('cancel', _('Canceled')),
    ('refund', _('Refunded')),
    ('partial_refund', _('Partially Refunded')),
    ('chargeback', _('Charged Back')),
    ('partial_chargeback', _('Partially Charged Back')),
    ('expire', _('Expired')),
    ('failure', _('Failed'))
)
PAYMENT_STATUSES = BILL_STATUSES
INVOICE_STATUSES = BILL_STATUSES
WITHDRAW_STATUSES = (
    ('pending', _('Pending')),
    ('process', _('On Process')),
    ('paid', _('Paid')),
    ('not-paid', _('Not Paid')),
    ("expire", _("Expired")),
)
TOPUP_STATUSES = (
    ("pending", _("Pending")),
    ("approve", _("Approved")),
    ("success", _("Transfered")),
    ("deny", _("Denied")),
    ("fail", _("Failed")),
    ("expire", _("Expired")),
)
SUBSCRIPTION_CHARGE_METHOD = (
    (1, _("Credit Card")),
    (2, _("Wallet"))
)
BANK_CHOICES = (
    ("ANGLOMAS", "Anglomas International Bank"),
    ("BANGKOK", "Bangkok Bank"),
    ("AGRIS", "Bank Agris"),
    ("AGRONIAGA", "Bank BRI Agroniaga"),
    ("AMAR", "Bank Amar Indonesia (formerly Anglomas International Bank)"),
    ("ANDARA", "Bank Andara"),
    ("ANZ", "Bank ANZ Indonesia"),
    ("ARTA_NIAGA_KENCANA", "Bank Arta Niaga Kencana"),
    ("ARTHA", "Bank Artha Graha International"),
    ("ARTOS", "Bank Artos Indonesia"),
    ("BISNIS_INTERNASIONAL", "Bank Bisnis Internasional"),
    ("BJB", "Bank BJB"),
    ("BJB_SYR", "Bank BJB Syariah"),
    ("BNI_SYR", "Bank BNI Syariah"),
    ("BNP_PARIBAS", "Bank BNP Paribas"),
    ("BUKOPIN", "Bank Bukopin"),
    ("BUMI_ARTA", "Bank Bumi Arta"),
    ("CAPITAL", "Bank Capital Indonesia"),
    ("BCA", "Bank Central Asia (BCA)"),
    ("BCA_SYR", "Bank Central Asia (BCA) Syariah"),
    ("CHINATRUST", "Bank Chinatrust Indonesia"),
    ("CIMB", "Bank CIMB Niaga"),
    ("COMMONWEALTH", "Bank Commonwealth"),
    ("DANAMON", "Bank Danamon"),
    ("DANAMON_UUS", "Bank Danamon UUS"),
    ("DBS", "Bank DBS Indonesia"),
    ("DINAR_INDONESIA", "Bank Dinar Indonesia"),
    ("DKI", "Bank DKI"),
    ("DKI_UUS", "Bank DKI UUS"),
    ("FAMA", "Bank Fama International"),
    ("GANESHA", "Bank Ganesha"),
    ("HANA", "Bank Hana"),
    ("HARDA_INTERNASIONAL", "Bank Harda Internasional"),
    ("HIMPUNAN_SAUDARA", "Bank Himpunan Saudara 1906"),
    ("ICBC", "Bank ICBC Indonesia"),
    ("INA_PERDANA", "Bank Ina Perdania"),
    ("INDEX_SELINDO", "Bank Index Selindo"),
    ("JASA_JAKARTA", "Bank Jasa Jakarta"),
    ("JTRUST", "Bank JTrust Indonesia (formerly Bank Mutiara)"),
    ("KESEJAHTERAAN_EKONOMI", "Bank Kesejahteraan Ekonomi"),
    ("MANDIRI", "Bank Mandiri"),
    ("MASPION", "Bank Maspion Indonesia"),
    ("MAYAPADA", "Bank Mayapada International"),
    ("MAYBANK", "Bank Maybank"),
    ("MAYBANK_SYR", "Bank Maybank Syariah Indonesia"),
    ("MAYORA", "Bank Mayora"),
    ("MEGA", "Bank Mega"),
    ("MESTIKA_DHARMA", "Bank Mestika Dharma"),
    ("MITRA_NIAGA", "Bank Mitra Niaga"),
    ("MIZUHO", "Bank Mizuho Indonesia"),
    ("MNC_INTERNASIONAL", "Bank MNC Internasional"),
    ("MUAMALAT", "Bank Muamalat Indonesia"),
    ("MULTI_ARTA_SENTOSA", "Bank Multi Arta Sentosa"),
    ("NATIONALNOBU", "Bank Nationalnobu"),
    ("BNI", "Bank Negara Indonesia (BNI)"),
    ("NUSANTARA_PARAHYANGAN", "Bank Nusantara Parahyangan"),
    ("OCBC", "Bank OCBC NISP"),
    ("OCBC_UUS", "Bank OCBC NISP UUS"),
    ("BAML", "Bank of America Merill-Lynch"),
    ("BOC", "Bank of China (BOC)"),
    ("INDIA", "Bank of India Indonesia"),
    ("TOKYO", "Bank of Tokyo Mitsubishi UFJ"),
    ("OKE", "Bank Oke Indonesia (formerly Bank Andara)"),
    ("PANIN", "Bank Panin"),
    ("PANIN_SYR", "Bank Panin Syariah"),
    ("PERMATA", "Bank Permata"),
    ("PERMATA_UUS", "Bank Permata UUS"),
    ("QNB_INDONESIA", "Bank QNB Indonesia (formerly Bank QNB Kesawan)"),
    ("RABOBANK", "Bank Rabobank International Indonesia"),
    ("BRI", "Bank Rakyat Indonesia (BRI)"),
    ("RESONA", "Bank Resona Perdania"),
    ("ROYAL", "Bank Royal Indonesia"),
    ("SAHABAT_SAMPOERNA", "Bank Sahabat Sampoerna"),
    ("SBI_INDONESIA", "Bank SBI Indonesia"),
    ("SHINHAN", "Bank Shinhan Indonesia (formerly Bank Metro Express)"),
    ("SINARMAS", "Bank Sinarmas"),
    ("SINARMAS_UUS", "Bank Sinarmas UUS"),
    ("MITSUI", "Bank Sumitomo Mitsui Indonesia"),
    ("BRI_SYR", "Bank Syariah BRI"),
    ("BUKOPIN_SYR", "Bank Syariah Bukopin"),
    ("MANDIRI_SYR", "Bank Syariah Mandiri"),
    ("MEGA_SYR", "Bank Syariah Mega"),
    ("BTN", "Bank Tabungan Negara (BTN)"),
    ("BTN_UUS", "Bank Tabungan Negara (BTN) UUS"),
    ("TABUNGAN_PENSIUNAN_NASIONAL", "Bank Tabungan Pensiunan Nasional"),
    ("UOB", "Bank UOB Indonesia"),
    ("VICTORIA_INTERNASIONAL", "Bank Victoria Internasional"),
    ("VICTORIA_SYR", "Bank Victoria Syariah"),
    ("WOORI", "Bank Woori Indonesia"),
    ("WOORI_SAUDARA", "Bank Woori Saudara Indonesia 1906 (formerly Bank Himpunan Saudara and Bank Woori Indonesia)"),
    ("YUDHA_BHAKTI", "Bank Yudha Bhakti"),
    ("ACEH", "BPD Aceh"),
    ("ACEH_UUS", "BPD Aceh UUS"),
    ("BALI", "BPD Bali"),
    ("BANTEN", "BPD Banten (formerly Bank Pundi Indonesia)"),
    ("BENGKULU", "BPD Bengkulu"),
    ("DAERAH_ISTIMEWA", "BPD Daerah Istimewa Yogyakarta (DIY)"),
    ("DAERAH_ISTIMEWA_UUS", "BPD Daerah Istimewa Yogyakarta (DIY) UUS"),
    ("JAMBI", "BPD Jambi"),
    ("JAMBI_UUS", "BPD Jambi UUS"),
    ("JAWA_TENGAH", "BPD Jawa Tengah"),
    ("JAWA_TENGAH_UUS", "BPD Jawa Tengah UUS"),
    ("JAWA_TIMUR", "BPD Jawa Timur"),
    ("JAWA_TIMUR_UUS", "BPD Jawa Timur UUS"),
    ("KALIMANTAN_BARAT", "BPD Kalimantan Barat"),
    ("KALIMANTAN_BARAT_UUS", "BPD Kalimantan Barat UUS"),
    ("KALIMANTAN_SELATAN", "BPD Kalimantan Selatan"),
    ("KALIMANTAN_SELATAN_UUS", "BPD Kalimantan Selatan UUS"),
    ("KALIMANTAN_TENGAH", "BPD Kalimantan Tengah"),
    ("KALIMANTAN_TIMUR", "BPD Kalimantan Timur"),
    ("KALIMANTAN_TIMUR_UUS", "BPD Kalimantan Timur UUS"),
    ("LAMPUNG", "BPD Lampung"),
    ("MALUKU", "BPD Maluku"),
    ("NUSA_TENGGARA_BARAT", "BPD Nusa Tenggara Barat"),
    ("NUSA_TENGGARA_BARAT_UUS", "BPD Nusa Tenggara Barat UUS"),
    ("NUSA_TENGGARA_TIMUR", "BPD Nusa Tenggara Timur"),
    ("PAPUA", "BPD Papua"),
    ("RIAU_DAN_KEPRI", "BPD Riau Dan Kepri"),
    ("RIAU_DAN_KEPRI_UUS", "BPD Riau Dan Kepri UUS"),
    ("SULAWESI", "BPD Sulawesi Tengah"),
    ("SULAWESI_TENGGARA", "BPD Sulawesi Tenggara"),
    ("SULSELBAR", "BPD Sulselbar"),
    ("SULSELBAR_UUS", "BPD Sulselbar UUS"),
    ("SULUT", "BPD Sulut"),
    ("SUMATERA_BARAT", "BPD Sumatera Barat"),
    ("SUMATERA_BARAT_UUS", "BPD Sumatera Barat UUS"),
    ("SUMSEL_DAN_BABEL", "BPD Sumsel Dan Babel"),
    ("SUMSEL_DAN_BABEL_UUS", "BPD Sumsel Dan Babel UUS"),
    ("SUMUT", "BPD Sumut"),
    ("SUMUT_UUS", "BPD Sumut UUS"),
    ("BTPN_SYARIAH", "BTPN Syariah (formerly BTPN UUS and Bank Sahabat Purba Danarta)"),
    ("CENTRATAMA", "Centratama Nasional Bank"),
    ("CCB", "China Construction Bank Indonesia (formerly Bank Antar Daerah and Bank Windu Kentjana International)"),
    ("CITIBANK", "Citibank"),
    ("DEUTSCHE", "Deutsche Bank"),
    ("HSBC_UUS", "Hongkong and Shanghai Bank Corporation (HSBC) UUS"),
    ("HSBC", "HSBC Indonesia (formerly Bank Ekonomi Raharja)"),
    ("EXIMBANK", "Indonesia Eximbank (formerly Bank Ekspor Indonesia)"),
    ("JPMORGAN", "JP Morgan Chase Bank"),
    ("MANDIRI_TASPEN", "Mandiri Taspen Pos (formerly Bank Sinar Harapan Bali)"),
    ("PRIMA_MASTER", "Prima Master Bank"),
    ("RBS", "Royal Bank of Scotland (RBS)"),
    ("STANDARD_CHARTERED", "Standard Chartered Bank"),
)



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: decimal_lib.py
# Project: core.ayopeduli.id
# File Created: Thursday, 1st November 2018 1:22:54 am
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Thursday, 1st November 2018 1:22:54 am
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Peduli sesama, sejahtera bersama
# Copyright - 2018 Ayopeduli.Id, ayopeduli.id
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from decimal import Decimal, ROUND_HALF_UP
TWOPLACES = Decimal(10) ** -2


def round_decimal(number):
    number = Decimal(number)
    return Decimal(number.quantize(Decimal('.01'), rounding=ROUND_HALF_UP))


def dec_to_str(amount_decimal):
    return str(amount_decimal.quantize(TWOPLACES))



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: view.py
# Project: django-enterprise-core
# File Created: Tuesday, 21st August 2018 11:51:25 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Tuesday, 21st August 2018 11:51:25 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-enterprise-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


import locale
import base64
import arrow
import datetime
from django.http import JsonResponse, HttpResponseForbidden
from django.core.exceptions import PermissionDenied
from django.views.defaults import permission_denied
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.contenttypes.models import ContentType
from django.conf import settings
from django.shortcuts import redirect, reverse
from django.views.decorators.csrf import csrf_exempt
from django.utils.decorators import method_decorator
from rest_framework.authtoken.models import Token
from ..brand import BrandManager
User = settings.AUTH_USER_MODEL



class ProtectedMixin(LoginRequiredMixin):
    """
    Mixin that will perform user access checking
    """
    login_url = "/authentication/login/"
    name_space = None
    model = None
    is_index_page = False

    def dispatch(self, request, *args, **kwargs):

        # If user not logged in, redirect to login page
        if not request.user.is_authenticated:
            return redirect(reverse("authentication:login") +
                            "?next=" + request.path_info)

        # If user is not staff nor super, 403 will be given
        if not self.is_staff(request):
            return self.handle_no_permission(request, *args, **kwargs)

        # set app and model access for user to the request
        app_access = []
        model_access = []
        for ct_int in request.user.groups.distinct(
                'permissions__content_type__model').values_list('permissions__content_type', flat=True):
            ct = ContentType.objects.filter(id=ct_int).first()
            if ct:
                if ct.app_label not in app_access:
                    app_access.append(ct.app_label)
                model_access.append("%s:%s" % (ct.app_label, ct.model))

        # Then store the app_access and model_access to the request
        request.app_access = app_access
        request.model_access = model_access
        request.permission_codenames = request.user.groups.values_list(
            'permissions__codename', flat=True)
        if not self.name_space:
            self.name_space = request.resolver_match.namespace
        if not self.model:
            self.model = request.resolver_match.url_name.replace(
                "_", "").split("-")[0]

        if not self.app_allowed(request):
            return self.handle_no_permission(request, *args, **kwargs)

        if not self.permission_allowed(request):
            return self.handle_no_permission(request, *args, **kwargs)

        return super(ProtectedMixin, self).dispatch(request, *args, **kwargs)

    def handle_no_permission(self, request, *args, **kwargs):
        if self.raise_exception:
            raise PermissionDenied(self.get_permission_denied_message())
        return permission_denied(
            request, "403: you're not authorized to access this app", template_name='home/403.html')

    def get_permissions(self, request, *args, **kwargs):
        return list(request.user.groups.distinct('permissions__codename').values_list(
            'permissions__codename', flat=True).all())

    def permission_allowed(self, request):
        if request.user.is_superuser:
            return True

        edit = request.GET.get("edit")
        if request.method == "GET":
            if "view_" + self.model in self.get_permissions(request):
                return True
            if self.is_index_page:
                return True
        elif edit and request.method == "POST":
            if "change_" + self.model in self.get_permissions(request):
                return True
        elif not edit and request.method == "POST":
            if "add_" + self.model in self.get_permissions(request):
                return True
        elif request.method == "DELETE":
            if "delete_" + self.model in self.get_permissions(request):
                return True
        elif edit and request.method == "PUT":
            if "approve_" + self.model in self.get_permissions(request):
                return True
            elif "publish_" + self.model in self.get_permissions(request):
                return True

        return False

    def app_allowed(self, request):
        if request.user.is_superuser:
            return True
        if self.name_space in request.app_access:
            return True
        if self.is_index_page:
            return True
        return False

    def is_staff(self, request):
        return (request.user.is_staff or request.user.is_superuser)


class BrandProtectedMixin(ProtectedMixin):
    brand = None

    def dispatch(self, request, *args, **kwargs):
        # If user not logged in, redirect to login page
        if not request.user.is_authenticated:
            return redirect(reverse("authentication:login") +
                            "?next=" + request.path_info)

        # If user not set the brand to interact with, redirect to set_brand
        # page
        if not request.session.get('brand'):
            return redirect(reverse("authentication:set_brand") +
                            "?next=" + request.path_info)

        brand_manager = BrandManager(user=request.user)
        brands = brand_manager.get_brands()

        if not brands:
            return self.handle_no_permission(request, *args, **kwargs)

        if not request.session.get(
                'brand') in brands.values_list('id62', flat=True):
            return self.handle_no_permission(request, *args, **kwargs)

        self.brand = brands.get(id62=request.session.get('brand'))
        request.brand = self.brand

        return super(BrandProtectedMixin, self).dispatch(
            request, *args, **kwargs)


class JSONResponseMixin(object):
    """
    A mixin that can be used to render a JSON response.
    """

    def render_to_json_response(self, context, **response_kwargs):
        """
        Returns a JSON response, transforming 'context' to make the payload.
        """
        return JsonResponse(
            self.get_data(context),
            **response_kwargs
        )

    def get_data(self, context):
        """
        Returns an object that will be serialized as JSON by json.dumps().
        """
        # Note: This is *EXTREMELY* naive; in reality, you'll need
        # to do much more complex handling to ensure that arbitrary
        # objects -- such as Django model instances or querysets
        # -- can be serialized as JSON.
        return context

    def handle_no_permission(self, request, *args, **kwargs):
        return permission_denied(
            request, "403: you're not authorized to access this app")



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: __init__.py
# Project: <<projectname>>
# File Created: Wednesday, 20th February 2019 3:00:43 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         http://ardz.xyz>
#
# Last Modified: Wednesday, 20th February 2019 3:00:43 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Crafted by Pro
# Copyright - <<year>> Ardz & Co, -
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.views.generic import TemplateView
from enterprise.libs.view import ProtectedMixin


class IndexView(ProtectedMixin, TemplateView):
    template_name = "log/index.html"

    def get(self, request):
        return self.render_to_response({})



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: urls.py
# Project: django-enterprise-core
# File Created: Tuesday, 21st August 2018 11:36:17 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Tuesday, 21st August 2018 11:36:17 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-enterprise-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.conf.urls import url, include
from .views import *

urlpatterns = [
    # url(r'^admin/', admin.site.urls),

    # login
    url(r'^login/$', LoginView.as_view(), name='login'),
    url(r'^login/success/$', LoginSuccessView.as_view(), name='login-success'),

    # logout
    url(r'^logout/$', LogoutView.as_view(), name='logout'),

    # change password
    url(r'^change-password/$',
        ChangePasswordView.as_view(),
        name='change-password'),
    url(r'^change-password/success/$',
        ChangePasswordSuccessView.as_view(),
        name='change-password-success'),

    url(r'^email_verify/$', EmailVerifyView.as_view(), name='email_verify'),
]



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: views.py
# Project: django-enterprise-core
# File Created: Tuesday, 21st August 2018 11:35:18 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Tuesday, 21st August 2018 11:35:58 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-enterprise-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.shortcuts import render
from django.http import HttpResponse
from django.views.generic import TemplateView
from django.utils.translation import gettext_lazy as _
from django.contrib.auth import login, logout
from django.shortcuts import redirect, get_object_or_404
from django.contrib import messages
from django.db import IntegrityError
from django import forms

from enterprise.libs.form import *
from enterprise.libs.view import ProtectedMixin
from enterprise.structures.authentication.models import EmailVerification


# Create your views here.
class LoginView(TemplateView):
    template_name = "login.html"

    def get(self, request):
        if request.user.is_authenticated:
            return redirect("authentication:login-success")

        return self.render_to_response({
            "form": AuthForm()
        })

    def post(self, request):
        next = request.GET.get("next")
        form = AuthForm(request.POST)

        if form.is_valid():
            login(request, form.cleaned_data['user'])
            if next:
                return redirect(next)
            else:
                return redirect("authentication:login-success")
        else:
            print(form.errors)
            return self.render_to_response({"form": form})


class LoginSuccessView(TemplateView):
    template_name = "login-success.html"

    def get(self, request):
        return self.render_to_response({})


class LogoutView(TemplateView):
    template_name = "logout.html"

    def get(self, request):
        logout(request)
        return self.render_to_response({})


class ChangePasswordView(ProtectedMixin, TemplateView):
    template_name = "change-password.html"

    def get(self, request):
        return self.render_to_response({
            "form": ChangePasswordForm(request.user)
        })

    def post(self, request):
        form = ChangePasswordForm(request.user, request.POST)

        if form.is_valid():
            form.save()
            messages.success(request, _('Your password has been changed.'))
            return redirect("authentication:change-password-success")
        else:
            return self.render_to_response({"form": form})


class ChangePasswordSuccessView(TemplateView):
    template_name = "change-password-success.html"

    def get(self, request):
        return self.render_to_response({})


class ResetPasswordView(TemplateView):

    template_name = 'reset-password.html'

    def get(self, request):
        reset_form = SetPasswordForm()
        return self.render_to_response({
            'reset_form': reset_form,
            'base_url': settings.BASE_URL,
        })

    def post(self, request):
        reset_form = SetPasswordForm(request.POST)
        if reset_form.is_valid():
            messages.success(
                request,
                _('Your reset password request has been processed, please check your email'))
        else:
            messages.error(request, form.errors)

        return self.render_to_response({
            'reset_form': reset_form,
            'base_url': settings.BASE_URL,
        })


class EmailVerifyView(TemplateView):
    template_name = "email_verify.html"

    def get(self, request):
        code = request.GET.get('c')
        # ev = get_object_or_404(EmailVerification, code=code, is_verified=False)
        ev = get_object_or_404(EmailVerification, code=code)
        user = get_object_or_404(User, id=ev.user_id)

        ev.is_verified = True
        ev.save()
        user.is_active = True
        user.save()

        set_password_form = SetPasswordForm(user) \
            if request.GET.get('is_reset_password') \
            else None

        register_form = RegisterForm() \
            if request.GET.get('is_acquire_account') \
            else None

        return self.render_to_response({
            'word': _('Dear %s, your email <%s> has been verified' % (user.full_name, ev.email)),
            'set_password_form': set_password_form,
            'register_form': register_form,
        })

    def post(self, request):
        code = request.GET.get('c')
        ev = get_object_or_404(EmailVerification, code=code)

        set_password_form = SetPasswordForm(ev.user, request.POST) \
            if request.GET.get('is_reset_password') \
            else None

        register_form = RegisterForm(request.POST) \
            if request.GET.get('is_acquire_account') \
            else None

        if set_password_form:
            if set_password_form.is_valid():
                set_password_form.save()
                messages.success(request, _('Your password has been set.'))
                return redirect("authentication:login")
            else:
                messages.error(request, set_password_form.errors)

        if register_form:
            if register_form.is_valid():
                ev.user.full_name = register_form.cleaned_data['full_name']
                ev.user.nick_name = ev.user.full_name
                ev.user.phone_number = register_form.cleaned_data['phone_number']

                try:
                    ev.user.save()
                    messages.success(
                        request, _('You\'re successfully registered.'))
                    return redirect("authentication:login")
                except IntegrityError as e:
                    messages.error(request, str(e).split(':')[-1])
            else:
                messages.error(request, register_form.errors)

        return self.render_to_response({
            'is_failed_set_password': True,
            'is_failed_register': True,
            'set_password_form': set_password_form,
            'register_form': register_form,
        })




# Django Module
from django.db import models

from ...common.models import BaseModelGeneric


class ResizeImageTemp(BaseModelGeneric):
    image = models.URLField(max_length=300)
    is_done = models.BooleanField(default=False)
    is_progress = models.BooleanField(default=False)
    purpose = models.CharField(max_length=100)

    def __str__(self):
        return self.image

    class Meta:
        verbose_name = "Resize Image Temp"
        verbose_name_plural = "Resize Image Temps"



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: __init__.py
# Project: django-panel-core
# File Created: Monday, 22nd April 2019 11:23:29 am
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Monday, 22nd April 2019 11:23:29 am
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-panel-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''



from django.db import models
from django.utils import timezone
from django.contrib.gis.geos import Point
from django.contrib.gis.db import models as geo
from django.db.models import Manager as GeoManager
from django.conf import settings

from enterprise.libs.moment import to_timestamp
from enterprise.libs.ip_address import get_client_ip

User = settings.AUTH_USER_MODEL


class Tracker(models.Model):
    created_at = models.DateTimeField(db_index=True, auto_now_add=True)
    created_at_timestamp = models.PositiveIntegerField(db_index=True)
    created_by = models.ForeignKey(User, on_delete=models.CASCADE, blank=True, null=True,
                                   related_name="%(app_label)s_%(class)s_created_by")
    modified_at = models.DateTimeField(auto_now=True)
    point = geo.PointField(null=True,)
    useragent = models.TextField(blank=True, null=True)
    ip = models.CharField(max_length=100, blank=True, null=True)
    duration = models.IntegerField(help_text="Duration in seconds", default=-1)
    trigger_action = models.CharField(max_length=100)
    visited_page = models.URLField(blank=True, null=True)
    OS = models.CharField(max_length=128, blank=True, null=True)
    referer = models.CharField(max_length=128, null=True, blank=True)
    tracking_id = models.CharField(max_length=100, null=True, blank=True)
    objects = GeoManager()
    log = models.TextField(blank=True, null=True)

    def save(self, *args, **kwargs):
        self.created_at = timezone.now()
        self.created_at_timestamp = to_timestamp(self.created_at)
        return super(Tracker, self).save(*args, **kwargs)

    def set_lat_lng(self, field_name, value):
        point = None

        if hasattr(
                self, field_name) and "longitude" in value and "latitude" in value:
            point = Point(value["longitude"], value["latitude"])
            setattr(self, field_name, point)
        return point

    class Meta:
        app_label = "tracker"


def create_tracker(request, trigger_action, is_get_or_create=False, log=None, os="web"):
    user = None
    if not request.user.is_anonymous:
        user = request.user

    useragent = request.META.get('HTTP_USER_AGENT', '')
    ip_address = get_client_ip(request)
    visited_page = request.META.get('HTTP_REFERER', None)

    tracker = None
    if is_get_or_create:
        tracker = Tracker.objects.filter(
            created_by=user,
            trigger_action=trigger_action,
            useragent=useragent)
        if not tracker.exists():
            tracker = Tracker()
        else:
            tracker = tracker.last()
    else:
        tracker = Tracker()

    tracker.created_by = user
    tracker.ip = ip_address
    tracker.useragent = useragent
    tracker.visited_page = visited_page
    tracker.trigger_action = trigger_action
    tracker.log = log
    tracker.OS = os
    tracker.save()



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: raw.py
# Project: <<projectname>>
# File Created: Wednesday, 20th February 2019 3:07:06 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         http://ardz.xyz>
#
# Last Modified: Wednesday, 20th February 2019 3:07:07 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Crafted by Pro
# Copyright - <<year>> Ardz & Co, -
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.views.generic import TemplateView

from datatable import Datatable

from ....libs.view import ProtectedMixin
from ....structures.tracker.models import Tracker


class RawView(ProtectedMixin, TemplateView):
    template_name = "log/raw.html"
    model_class = Tracker

    def get(self, request):
        if request.GET.get('draw', None) is not None:
            return self.datatable(request)

        return self.render_to_response({
            'title': "Tracker enterprise - %s" % self.model_class.__name__,
        })

    def datatable(self, request):
        qs = self.model_class.objects.all()
        defer = ['created_at', 'created_by', 'visited_page', 'referer', 'ip']

        d = Datatable(request, qs, defer, key="id")
        d.set_lookup_defer(['created_by__stage_name'])

        return d.get_data()



import json
from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from django.conf import settings

from enterprise.libs.moment import to_timestamp
from enterprise.libs import storage
from enterprise.structures.common.models.base import BaseModelGeneric


User = settings.AUTH_USER_MODEL

# Create your models here.


class Log(models.Model):
    content_type = models.ForeignKey(ContentType,
                                     related_name="%(app_label)s_%(class)s_content_type",
                                     blank=True, null=True, on_delete=models.CASCADE,)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    logged_at = models.DateTimeField(blank=True, null=True)
    logged_at_timestamp = models.PositiveIntegerField(blank=True, null=True)
    logged_by = models.ForeignKey(
        User, db_index=True, on_delete=models.CASCADE,)

    def __str__(self):
        return self.logged_by.__str__()

    def get_message_dict(self):
        try:
            return json.loads(self.message)
        except BaseException:
            return {}

    def get_note(self):
        message_dict = self.get_message_dict()
        if "note" in message_dict:
            return message_dict['note']
        else:
            return "-"

    def save(self, *args, **kwargs):
        self.logged_at = timezone.now()
        self.logged_at_timestamp = to_timestamp(self.logged_at)

        return super(Log, self).save(*args, **kwargs)

    def read(self, *args, **kwargs):
        self.read_at = timezone.now()
        self.read_at_timestamp = to_timestamp(self.read_at)

        return self.save(*args, **kwargs)

    class Meta:
        verbose_name = _("Log")
        verbose_name_plural = _("Logs")


class APILog(models.Model):
    """
    For API log each call, each user
    """
    created_at = models.DateTimeField(auto_now_add=True)
    modified_at = models.DateTimeField(auto_now=True)
    user = models.ForeignKey(
        User,
        blank=True,
        null=True,
        on_delete=models.CASCADE)

    app_id = models.CharField(max_length=100)
    uid = models.CharField(max_length=100, blank=True, null=True)
    latitude = models.CharField(max_length=100)
    longitude = models.CharField(max_length=100)
    endpoint = models.CharField(max_length=100)
    request_method = models.CharField(max_length=10)
    city = models.CharField(max_length=50)
    province = models.CharField(max_length=50)
    country = models.CharField(max_length=50)

    def __str__(self):
        return self.app_id

    class Meta:
        verbose_name = _("APILog")
        verbose_name_plural = _("APILogs")


class File(BaseModelGeneric):
    display_name = models.CharField(max_length=150)
    short_name = models.SlugField(max_length=150)
    file = models.FileField(
        storage=storage.FILE_STORAGE,
        max_length=300,
        blank=True,
        null=True
    )
    description = models.TextField(blank=True, null=True)
    content_type = models.ForeignKey(
        ContentType,
        on_delete=models.CASCADE,
        blank=True,
        null=True)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    def __str__(self):
        return self.display_name

    def get_file(self):
        if self.file:
            return self.file.url
        return None

    def get_safe_url(self):
        ru = settings.RACKSPACE_BASE_URL + '/'
        url = self.file.url
        if 'http' in url:
            url = url.replace(ru,'')
        if not 'http' in url:
            url = ru + url
        return url

    class Meta:
        verbose_name = _("File")
        verbose_name_plural = _("Files")



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: admin.py
# Project: core.ayopeduli.id
# File Created: Thursday, 1st November 2018 1:48:05 am
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Thursday, 1st November 2018 1:48:06 am
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Peduli sesama, sejahtera bersama
# Copyright - 2018 Ayopeduli.Id, ayopeduli.id
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.contrib import admin
from .models import *


class WalletAdmin(admin.ModelAdmin):
    pass


class InvoiceAdmin(admin.ModelAdmin):
    pass


class InvoiceItemAdmin(admin.ModelAdmin):
    pass


class TopUpAdmin(admin.ModelAdmin):
    pass


admin.site.register(Wallet, WalletAdmin)
admin.site.register(Invoice, InvoiceAdmin)
admin.site.register(InvoiceItem, InvoiceItemAdmin)
admin.site.register(TopUp, TopUpAdmin)



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: midtrans.py
# Project: core.wecare.id
# File Created: Monday, 7th January 2019 11:03:02 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Monday, 7th January 2019 11:03:02 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Handcrafted and Made with Love
# Copyright - 2018 Wecare.Id, wecare.id
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.db import models
from django.utils.translation import gettext_lazy as _
from django.contrib.postgres.fields import JSONField
from enterprise.libs.payment.midtrans import PAYMENT_STATUSES, PAYMENT_TYPE_CHOICES
from enterprise.structures.common.models import BaseModelGeneric


class Midtrans(BaseModelGeneric):
    payment_type = models.CharField(
        choices=PAYMENT_TYPE_CHOICES, max_length=40)
    transaction_status = models.CharField(
        choices=PAYMENT_STATUSES, max_length=40)
    transaction_id = models.CharField(max_length=40, blank=True, null=True)
    amount = models.DecimalField(default=0, decimal_places=2, max_digits=12)
    payload = JSONField(blank=True, null=True)
    responses = JSONField(blank=True, null=True)

    def __str__(self):
        return "%s : %s <%s>" % (
            self.created_by,
            self.amount,
            self.transaction_status
        )

    class Meta:
        verbose_name = _("Midtrans")
        verbose_name_plural = _("Midtranses")



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: urls.py
# Project: django-enterprise-core
# File Created: Wednesday, 22nd August 2018 12:42:51 am
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Wednesday, 22nd August 2018 12:42:51 am
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-enterprise-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.conf.urls import url, include
from .views import IndexView
from .views.user import UserView, UserFormView
from .views.group import GroupView, GroupFormView
from .views.permission import PermissionView, PermissionFormView

urlpatterns = [
    url(r'^$', IndexView.as_view(), name='index'),

    # User
    url(r'^user/$', UserView.as_view(), name='user'),
    url(r'^user/form/$', UserFormView.as_view(), name='user-form'),

    # Group
    url(r'^group/$', GroupView.as_view(), name='group'),
    url(r'^group/form/$', GroupFormView.as_view(), name='group-form'),

    # Permission
    url(r'^permission/$', PermissionView.as_view(), name='permission'),
    url(r'^permission/form/$',
        PermissionFormView.as_view(),
        name='permission-form'),
]



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: manual.py
# Project: <<projectname>>
# File Created: Friday, 24th May 2019 9:16:46 pm
# 
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         http://ardz.xyz>
# 
# Last Modified: Friday, 24th May 2019 9:16:47 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
# 
# Crafted by Pro
# Copyright - <<year>> Ardz & Co, -
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.db import models
from django.utils.translation import gettext_lazy as _
from django.contrib.postgres.fields import JSONField

from enterprise.libs.payment.midtrans import PAYMENT_STATUSES
from enterprise.libs import storage
from enterprise.structures.common.models import BaseModelGeneric


class Manual(BaseModelGeneric):
    transaction_status = models.CharField(
        choices=PAYMENT_STATUSES, max_length=40, default='pending')
    amount = models.DecimalField(default=0, decimal_places=2, max_digits=12)
    attachement = models.FileField(
        storage=storage.FILE_STORAGE,
        max_length=300,
        blank=True,
        null=True
    )

    def __str__(self):
        return "%s : %s <%s>" % (
            self.created_by,
            self.amount,
            self.transaction_status
        )

    class Meta:
        verbose_name = _("Manual")
        verbose_name_plural = _("Manuals")


'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: __init__.py
# Project: <<projectname>>
# File Created: Wednesday, 20th February 2019 2:54:01 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         http://ardz.xyz>
#
# Last Modified: Wednesday, 20th February 2019 2:54:01 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Crafted by Pro
# Copyright - <<year>> Ardz & Co, -
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: __init__.py
# Project: core.lakon.app
# File Created: Wednesday, 5th September 2018 5:06:38 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Wednesday, 5th September 2018 5:06:38 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Lakon, lakon.app
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: __init__.py
# Project: core.ayopeduli.id
# File Created: Thursday, 1st November 2018 1:16:00 am
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Thursday, 1st November 2018 1:16:05 am
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Peduli sesama, sejahtera bersama
# Copyright - 2018 Ayopeduli.Id, ayopeduli.id
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


import decimal
from datetime import date
from dateutil.relativedelta import relativedelta
from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver
from django.utils import timezone
from django.utils.translation import gettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.postgres.fields import ArrayField
from django.conf import settings

from enterprise.libs import base36
from enterprise.libs.moment import to_timestamp
from enterprise.libs.decimal_lib import dec_to_str
from enterprise.libs.pay_constants import (WITHDRAW_STATUSES, INVOICE_STATUSES,
                                      PAYMENT_STATUSES, TOPUP_STATUSES)
from enterprise.structures.common.models import BaseModelGeneric

from .midtrans import Midtrans
from .manual import Manual

User = settings.AUTH_USER_MODEL


class Wallet(BaseModelGeneric):
    amount = models.DecimalField(max_digits=19, decimal_places=2)
    description = models.CharField(max_length=200, blank=True, null=True)
    content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, blank=True, null=True)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    def __str__(self):
        return "%s:%s" % (self.created_by, self.amount)

    def get_formatted_amount(self):
        return 'Rp.{:,.0f},-'.format(self.amount)

    def get_absolute_formatted_amount(self):
        return 'Rp.{:,.0f},-'.format(abs(self.amount))

    class Meta:
        verbose_name = _("Wallet")
        verbose_name_plural = _("Wallet")


class Invoice(BaseModelGeneric):
    number = models.CharField(max_length=20)
    amount = models.DecimalField(max_digits=19, decimal_places=2)
    status = models.CharField(
        max_length=20, choices=INVOICE_STATUSES, default='pending')

    def __str__(self):
        return str(self.number)

    def create_number(self, *args, **kwargs):
        if not self.number and self.pk:
            CONSTANT_START_NUMBER = 11470
            number = self.pk + CONSTANT_START_NUMBER
            self.number = '{}{}'.format("I", base36.encode(number))
            self.save()

    def get_items(self):
        return InvoiceItem.objects.filter(
            deleted_at__isnull=True,
            invoice=self
        ).all()

    def get_status(self):
        return dict(INVOICE_STATUSES)[self.status]

    def save(self, *args, **kwargs):
        su = super(Invoice, self).save(*args, **kwargs)
        self.create_number()
        return su

    def get_formatted_amount(self):
        return 'Rp.{:,.0f},-'.format(self.amount)

    class Meta:
        verbose_name = _("Invoice")
        verbose_name_plural = _("Invoices")


class InvoiceItem(BaseModelGeneric):
    invoice = models.ForeignKey(Invoice, on_delete=models.CASCADE)
    amount = models.DecimalField(max_digits=19, decimal_places=2)
    qty = models.PositiveIntegerField(default=1)
    name = models.CharField(max_length=50, blank=True, null=True)
    content_type = models.ForeignKey(
        ContentType, on_delete=models.CASCADE, blank=True, null=True)
    object_id = models.PositiveIntegerField(blank=True, null=True)
    content_object = GenericForeignKey('content_type', 'object_id')

    def __str__(self):
        return "%s:%s" % (self.name, self.get_total_amount())

    def get_total_amount(self):
        return decimal.Decimal(self.qty) * self.amount

    def get_formatted_total_amount(self):
        return 'Rp.{:,.0f},-'.format(self.get_total_amount())

    def get_formatted_amount(self):
        return 'Rp.{:,.0f},-'.format(self.amount)

    class Meta:
        verbose_name = _("Invoice Item")
        verbose_name_plural = _("Invoice Items")


class TopUp(BaseModelGeneric):
    amount = models.DecimalField(max_digits=19, decimal_places=2)
    invoice = models.ForeignKey(Invoice, on_delete=models.CASCADE)
    midtrans = models.ForeignKey(
        Midtrans,
        on_delete=models.CASCADE,
        blank=True, null=True
    )
    manual = models.ForeignKey(
        Manual,
        on_delete=models.CASCADE,
        blank=True, null=True
    )
    expired_at = models.DateTimeField(blank=True, null=True)
    status = models.CharField(
        max_length=20, choices=TOPUP_STATUSES, default='pending')

    def __str__(self):
        return "%s - %s" % (self.created_by, self.amount, )

    def approve(self, *args, **kwargs):
        self.status = 'approve'
        self.save(*args, **kwargs)

    def deny(self, *args, **kwargs):
        self.status = 'deny'
        self.save(*args, **kwargs)

    def get_formatted_amount(self):
        return 'Rp.{:,.0f},-'.format(self.amount)

    class Meta:
        verbose_name = _("Top Up")
        verbose_name_plural = _("Top Up")


# Bank
class Bank(BaseModelGeneric):
    display_name = models.CharField(max_length=100)
    short_name = models.SlugField(max_length=100)
    code = models.SlugField(max_length=100)

    def __str__(self):
        return self.display_name

    class Meta:
        verbose_name = _("Bank")
        verbose_name_plural = _("Banks")


class BankAccount(BaseModelGeneric):
    bank = models.ForeignKey(Bank, on_delete=models.CASCADE)
    number = models.CharField(max_length=20)
    name = models.CharField(max_length=120)
    notes = models.TextField(blank=True, null=True)  # branch name, etc.d

    def __str__(self):
        return '%s (%s:%s)' % (self.name, self.bank.__str__(), self.number)

    class Meta:
        verbose_name = _("Bank Account")
        verbose_name_plural = _("Bank Accounts")


class Withdraw(BaseModelGeneric):
    # use : created_by, approved_by; created_by = requester, approved_by = managed_by
    balance = models.DecimalField(max_digits=19, decimal_places=2)
    amount = models.DecimalField(max_digits=19, decimal_places=2)
    status = models.CharField(
        max_length=20, choices=WITHDRAW_STATUSES, default="pending")
    bank_account = models.ForeignKey(BankAccount, on_delete=models.CASCADE)
    expired_at = models.DateTimeField(blank=True, null=True)

    def __str__(self):
        return "%s:%s" % (self.get_formatted_amount(), self.created_by)

    def get_formatted_amount(self):
        return 'Rp.{:,.0f},-'.format(self.amount)

    def get_formatted_balance(self):
        return 'Rp.{:,.0f},-'.format(self.balance)

    class Meta:
        verbose_name = _("Withdraw")
        verbose_name_plural = _("Withdraws")


@receiver(post_save, sender=TopUp)
def do_topup(sender, instance, created, **kwargs):
    from enterprise.libs.payment.wallet import topup_wallet, transfer_wallet

    if instance.status == "approve":
        topup_wallet(
            topup=instance,
            description="TopUp wallet <invoice: %s>" % instance.invoice.number
        )

        for item in instance.invoice.get_items():
            print(item)
            print("transfering balance")
            if item.content_object:
                receiver = item.content_object.owned_by
                transfer_wallet(
                    instance.owned_by,
                    receiver,
                    int(item.amount),
                    obj=item,
                    description='%s: %s' % (
                        item.content_type.__str__(),
                        item.name
                    )
                )
        instance.status = 'success'
        instance.save()


def get_default_data():
    return [1]



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: rackspace_cloud_files.py
# Project: enterprise.ayopeduli.id
# File Created: Wednesday, 31st October 2018 11:12:48 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Wednesday, 31st October 2018 11:12:49 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Peduli sesama, sejahtera bersama
# Copyright - 2018 Ayopeduli.Id, ayopeduli.id
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


import os
from rackspace import connection
from django.conf import settings
from django.utils import timezone
from django.core.files.base import ContentFile
from django.core.files.storage import FileSystemStorage
from django.utils.crypto import get_random_string
from django.utils.deconstruct import deconstructible
from django.utils.encoding import force_text as force_unicode, smart_str
from django.utils.functional import cached_property

from django.contrib.auth import get_user_model

"""
https://developer.openstack.org/sdks/python/openstacksdk/users/resource.html#openstack.resource.Resource
https://developer.openstack.org/sdks/python/openstacksdk/users/resources/object_store/v1/obj.html#openstack.object_store.v1.obj.Object
https://developer.openstack.org/sdks/python/openstacksdk/users/proxies/object_store.html
https://developer.openstack.org/sdks/python/openstacksdk/users/guides/object_store.html

conn.object_store
['bulk_delete', 'containers', 'copy_object', 'create_container', 'delete_account_metadata', 'delete_container', 'delete_container_metadata', 'delete_object', 'delete_object_metadata', 'download_object', 'get_account_metadata', 'get_container_metadata', 'get_object', 'get_object_metadata', 'objects', 'set_account_metadata', 'set_container_metadata', 'set_object_metadata', 'upload_object', 'wait_for_delete', 'wait_for_status']

conn
['alarm', 'authenticator', 'authorize', 'backup', 'bare_metal', 'block_store', 'cluster', 'compute', 'database', 'identity', 'image', 'key_manager', 'load_balancer', 'message', 'monitoring', 'network', 'object_store', 'orchestration', 'profile', 'session', 'telemetry', 'workflow']

container
['allow_create', 'allow_delete', 'allow_head', 'allow_list', 'allow_retrieve', 'allow_update',
'base_path', 'bytes', 'bytes_used',
'clear', 'content_type', 'convert_ids', 'count', 'create', 'create_by_id',
'delete', 'delete_by_id', 'delete_metadata',
'existing', 'find', 'from_id', 'from_name',
'get', 'get_by_id', 'get_data_by_id', 'get_headers', 'get_id', 'get_resource_name',
'head', 'head_by_id', 'head_data_by_id',
'id', 'id_attribute', 'if_none_match', 'is_content_type_detected', 'is_dirty', 'is_newest', 'items', 'iteritems', 'iterkeys', 'itervalues',
'keys',
'list', 'location',
'metadata',
'name', 'name_attribute',
'new',
'object_count', 'patch_update', 'pop', 'popitem',
'read_ACL', 'resource_key', 'resource_name', 'resources_key',
'service', 'set_headers', 'set_metadata', 'setdefault', 'sync_key', 'sync_to',
'timestamp', 'to_dict', 'update', 'update_attrs', 'update_by_id', 'values', 'versions_location', 'write_ACL']

obj
[
'accept_ranges', 'allow_create', 'allow_delete', 'allow_head', 'allow_list', 'allow_retrieve', 'allow_update',
'base_path', 'bytes', 'clear', 'container', 'content_disposition', 'content_encoding', 'content_length', 'content_type', 'convert_ids', 'copy_from', 'create', 'create_by_id', 'data', 'delete', 'delete_after', 'delete_at', 'delete_by_id', 'delete_metadata', 'etag', 'existing', 'expires_at', 'find', 'from_id', 'from_name', 'get', 'get_by_id', 'get_data_by_id', 'get_headers', 'get_id', 'get_resource_name', 'hash', 'head', 'head_by_id', 'head_data_by_id', 'id', 'id_attribute', 'if_match', 'if_modified_since', 'if_none_match', 'if_unmodified_since', 'is_content_type_detected', 'is_dirty', 'is_newest', 'is_static_large_object', 'items', 'iteritems', 'iterkeys', 'itervalues', 'keys', 'last_modified_at', 'list', 'location', 'metadata', 'multipart_manifest', 'name', 'name_attribute', 'new', 'object_manifest', 'patch_update', 'pop', 'popitem', 'range', 'resource_key', 'resource_name', 'resources_key', 'service', 'set_headers', 'set_metadata', 'setdefault', 'signature', 'timestamp', 'to_dict', 'transfer_encoding', 'update', 'update_attrs', 'update_by_id', 'values']
"""
conn = connection.Connection(
    username=settings.RACKSPACE_CLOUD_FILES["username"],
    api_key=settings.RACKSPACE_CLOUD_FILES["key"],
    region=settings.RACKSPACE_CLOUD_FILES["region"]
)


@deconstructible
class RackspaceStorage(FileSystemStorage):
    def __init__(self, *args, **kwargs):
        if not settings.USE_RACKSPACE:
            return None

        default_container = kwargs.get(
            "container", settings.RACKSPACE_CLOUD_FILES["default_container"])
        self.purpose = kwargs.pop("purpose")
        self.container = conn.object_store.get_container_metadata(
            default_container)
        super(RackspaceStorage, self).__init__(*args, **kwargs)

    def _open(self, name, mode='rb'):
        # This must return a File object
        return ContentFile(conn.object_store.get_object(
            name, container=self.container))

    def _save(self, name, content, encode_name=True):
        # Should return the actual name of name of the file saved (usually the
        # name passed in, but if the storage needs to change the file name
        # return the new name instead).
        if encode_name:
            name = self.get_valid_name(name)

        uploaded = conn.object_store.upload_object(
            container=self.container,
            name=name,
            data=content
        )

        # resize here, send to celery
        if self.purpose and hasattr(uploaded, "name"):
            from enterprise.structures.integration.models import ResizeImageTemp
            rit = ResizeImageTemp(
                image=self.url(
                    uploaded.name),
                purpose=self.purpose)
            rit.created_by = get_user_model().objects.first()
            rit.save()

        return name

    @cached_property
    def location(self):
        return self._location

    @cached_property
    def base_url(self):
        if self._base_url is not None and not self._base_url.endswith('/'):
            self._base_url += '/'
        return self._base_url

    def get_valid_name(self, name):
        form = "%Y-%m-%d"
        extension = os.path.splitext(name)[1]
        date = os.path.normpath(
            force_unicode(
                timezone.now().strftime(
                    smart_str(form)
                )
            )
        )
        rand_name = '%s%s' % (get_random_string(), extension)
        return '%s%s/%s' % (self.location, date, rand_name)

    def delete(self, name):
        # delete object
        conn.object_store.delete_object(
            name,
            ignore_missing=settings.DEBUG,
            container=self.container
        )

    def exists(self, name):
        return self.size(name) > 0

    def size(self, name):
        try:
            obj = conn.object_store.get_object_metadata(
                name, container=self.container)
        except BaseException:
            return 0
        return obj.content_length

    def url(self, name):
        return "%s%s" % (self.base_url, name)



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: base62.py
# Project: django-enterprise-core
# File Created: Thursday, 16th August 2018 11:50:21 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Thursday, 16th August 2018 11:50:21 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-enterprise-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


# http://stackoverflow.com/questions/561486/how-to-convert-an-integer-to-the-shortest-url-safe-string-in-python?lq=1
import string

ALPHABET = string.ascii_uppercase + string.ascii_lowercase + string.digits
ALPHABET_REVERSE = dict((c, i) for (i, c) in enumerate(ALPHABET))
BASE = len(ALPHABET)
SIGN_CHARACTER = '$'


def base62_encode(n):
    if n < 0:
        return SIGN_CHARACTER + num_encode(-n)

    s = []

    while True:
        n, r = divmod(n, BASE)
        s.append(ALPHABET[r])

        if n == 0:
            break

    return ''.join(reversed(s))


def base62_decode(s):
    if s[0] == SIGN_CHARACTER:
        return -num_decode(s[1:])

    n = 0

    for c in s:
        n = n * BASE + ALPHABET_REVERSE[c]

    return n



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: tracker.py
# Project: <<projectname>>
# File Created: Tuesday, 19th February 2019 3:00:02 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         http://ardz.xyz>
#
# Last Modified: Tuesday, 19th February 2019 3:00:02 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Crafted by Pro
# Copyright - <<year>> Ardz & Co, -
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.conf import settings
from enterprise.libs.ip_address import get_client_ip


class TrackerMixin(object):
    def dispatch(self, request, *args, **kwargs):
        user = None
        if not request.user.is_anonymous:
            user = request.user

        tracking_id = request.GET.get('tracking_id')
        if not tracking_id:
            tracking_id = request.GET.get('track_id')
        if not tracking_id:
            tracking_id = request.GET.get('t_id')

        referer = request.META.get('HTTP_REFERER')
        useragent = request.META.get('HTTP_USER_AGENT')
        ip_address = get_client_ip(request)
        visited_page = "%s%s" % (
            request.META.get('HTTP_HOST'),
            request.get_full_path()
        )

        Tracker = self.get_model()
        tracker = Tracker()

        tracker.created_by = user
        tracker.ip = ip_address
        tracker.useragent = useragent[0:128] if useragent else None
        tracker.referer = referer[0:128] if referer else None
        tracker.trigger_action = request.method
        tracker.tracking_id = tracking_id
        tracker.visited_page = visited_page.split('?')[0]
        tracker.save()

        request.fb_app_id = getattr(settings, 'SOCIAL_AUTH_FACEBOOK_KEY')

        return super().dispatch(request, *args, **kwargs)

    def get_model(self):
        from ...structures.tracker.models import Tracker
        return Tracker



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: base36.py
# Project: core.ayopeduli.id
# File Created: Thursday, 1st November 2018 1:30:29 am
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Thursday, 1st November 2018 1:30:29 am
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Peduli sesama, sejahtera bersama
# Copyright - 2018 Ayopeduli.Id, ayopeduli.id
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


BASE36l = "0123456789abcdefghijklmnopqrstuvwxyz"
BASE36u = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"


def encode(num, alphabet=BASE36l):
    """Encode a positive number in Base X

    Arguments:
    - `num`: The number to encode
    - `alphabet`: The alphabet to use for encoding
    """
    if num == 0:
        return alphabet[0]
    arr = []
    base = len(alphabet)
    while num:
        num, rem = divmod(num, base)
        arr.append(alphabet[rem])
    arr.reverse()
    return ''.join(arr)


def decode(string, alphabet=BASE36l):
    """Decode a Base X encoded string into the number

    Arguments:
    - `string`: The encoded string
    - `alphabet`: The alphabet to use for encoding
    """
    base = len(alphabet)
    strlen = len(string)
    num = 0

    idx = 0
    for char in string:
        power = (strlen - (idx + 1))
        num += alphabet.index(char) * (base ** power)
        idx += 1

    return num



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: __init__.py
# Project: django-enterprise-core
# File Created: Wednesday, 22nd August 2018 12:45:04 am
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Wednesday, 22nd August 2018 12:45:05 am
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Ardz Co, https://github.com/ardzix/django-enterprise-core
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''
from django.views.generic import TemplateView
from enterprise.libs.view import ProtectedMixin


class IndexView(ProtectedMixin, TemplateView):
    template_name = "superuser/index.html"

    def get(self, request):
        return self.render_to_response({})



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: ip_address.py
# Project: core.wecare.id
# File Created: Tuesday, 27th November 2018 3:44:01 am
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Tuesday, 27th November 2018 3:44:01 am
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Handcrafted and Made with Love
# Copyright - 2018 Wecare.Id, wecare.id
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


def get_client_ip(request):
    x_forwarded_for = request.META.get('HTTP_X_FORWARDED_FOR')
    if x_forwarded_for:
        ip_list = x_forwarded_for.split(',')
        ip = ip_list[len(ip_list) - 2] if len(ip_list) >= 2 else ip_list[0]
    else:
        ip = request.META.get('REMOTE_ADDR')
    return ip



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: wallet.py
# Project: <<projectname>>
# File Created: Friday, 24th May 2019 9:10:35 pm
# 
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         http://ardz.xyz>
# 
# Last Modified: Friday, 24th May 2019 9:10:35 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
# 
# Crafted by Pro
# Copyright - <<year>> Ardz & Co, -
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''

import datetime
from decimal import Decimal
from django.db.models import Sum
from ...structures.transaction.models import Wallet


def get_balance(user):
    balance = Wallet.objects.filter(
        owned_by=user, deleted_at__isnull=True,
        ).aggregate(Sum('amount'))['amount__sum']
    return balance if balance is not None else Decimal(0)

def transfer_wallet(from_user, to_user, total, obj=None, description="Transfer balance"):

    if total <= 0:
        raise Exception("Total transfer must be greater than zero")

    if from_user == to_user:
        raise Exception("Cannot transfer to same user (user:%s, obj:%s)" %
                (
                    str(from_user),
                    str(obj) if obj else '-'
                )
            )

    current_balance = get_balance(from_user)
    if current_balance < total:
        raise Exception("Insufficient balance %d:%d" % (current_balance, total))

    # deduct first
    wallet = Wallet()
    wallet.created_by = from_user
    wallet.amount = total * -1 if total > 0 else total
    wallet.description = description
    if obj:
        wallet.content_type = obj.get_content_type()
        wallet.object_id = obj.id
    wallet.save()
    print("Deduction success")

    # add credit
    wallet = Wallet()
    wallet.created_by = to_user
    wallet.amount = total * -1 if total < 0 else total
    wallet.description = description
    if obj:
        wallet.content_type = obj.get_content_type()
        wallet.object_id = obj.id
    wallet.save()
    print("Addition Success")
    print("Current balance now %s" % str(current_balance - total))
    return True

def topup_wallet(topup, description="Topup balance"):

    wallet = Wallet.objects.create(
        created_by = topup.owned_by,
        content_type = topup.get_content_type(),
        object_id = topup.id,
        amount = topup.amount
    )
    wallet.description = description
    wallet.save()
    print("Topup Success for %s" % wallet.description)
    return True

def deduct_wallet(user, amount, obj=None, description="Deduction"):
    current_balance = get_balance(user)
    if current_balance < amount:
        raise Exception("Insufficient balance %d:%d" % (current_balance, amount))

    # deduct first
    wallet = Wallet()
    wallet.created_by = user
    wallet.amount = amount * -1 if amount > 0 else amount
    wallet.description = description
    if obj:
        wallet.content_type = obj.get_content_type()
        wallet.object_id = obj.id
    wallet.save()
    print("Deduction success (%s)" % amount)
    print(wallet)
    return True


class WalletChannelManager(object):
    invoice = None

    def __init__(self, *args, **kwargs):
        self.invoice = kwargs.get('invoice')

    def charge(self, user, amount):
        if not self.invoice:
            raise Exception("Invoice not provided")

        if self.invoice.status == 'settlement':
            raise Exception("Invoice already paid")

        items = []
        for item in self.invoice.get_items():
            items.append({
                'name' : item.name,
                'qty' : item.qty,
                'amount' : item.amount,
            })

            receiver = item.content_object.owned_by

            transfer_wallet(
                user, 
                receiver, 
                int(item.amount * item.qty),
                obj = item,
                description = '%s: %s%s' % (
                    item.content_type.__str__(),
                    item.name,
                    ' x%d' % item.qty if item.qty >1 else ''
                )
            )

        self.invoice.status = 'settlement'
        self.invoice.updated_by = user
        self.invoice.save()

        return {
            'message': 'Transfer success',
            'items':items
        }


'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: __init__.py
# Project: core.wecare.id
# File Created: Monday, 7th January 2019 10:15:02 pm
# 
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
# 
# Last Modified: Monday, 7th January 2019 10:15:03 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
# 
# Handcrafted and Made with Love
# Copyright - 2018 Wecare.Id, wecare.id
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from .midtrans import Gopay, BankTransfer, MandiriEChannel, CreditCard, Snap

class PaymentManager(object):
    channel = 'wallet'
    channel_manager = None

    def __init__(self, channel=None, *args, **kwargs):
        if channel:
            self.channel = channel

        if channel == 'gopay':
            self.channel_manager = Gopay(
                callback_url=kwargs.get('callback_url'), 
                invoice=kwargs.get('invoice')
            )
        elif channel == 'permata' or channel == 'bca' or channel == 'bni':
            self.channel_manager = BankTransfer(
                invoice=kwargs.get('invoice'),
                bank=channel
            )
        elif channel == 'mandiri':
            self.channel_manager = MandiriEChannel(invoice=kwargs.get('invoice'))
        elif channel == 'credit_card':
            self.channel_manager = CreditCard(
                invoice=kwargs.get('invoice'),
                card=kwargs.get('card'),
            )
        elif channel == 'wallet':
            from ..wallet import WalletChannelManager
            self.channel_manager = WalletChannelManager(invoice=kwargs.get('invoice'))
        elif channel == 'snap':
            self.channel_manager = Snap(
                invoice=kwargs.get('invoice'), 
                payment_channel=kwargs.get('payment_channel')
            )
        else:
            raise Exception("Channel not available")

    def get_channel_manager(self):
        return self.channel_manager


'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: user.py
# Project: <<projectname>>
# File Created: Monday, 10th December 2018 2:39:06 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         http://ardz.xyz>
#
# Last Modified: Monday, 10th December 2018 2:39:06 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Crafted by Pro
# Copyright - <<year>> Ardz & Co, -
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


from django.views.generic import TemplateView
from django.shortcuts import redirect
from django.utils.translation import gettext_lazy as _
from django.contrib import messages
from django.contrib.auth import get_user_model
from enterprise.libs.view import ProtectedMixin
from datatable import Datatable
from ..forms import UserForm


class UserView(ProtectedMixin, TemplateView):
    template_name = "superuser/user.html"

    def get(self, request, *args, **kwargs):

        if request.GET.get('draw', None) is not None:
            return self.datatable(request)

        return self.render_to_response({
            'title': "Tracker enterprise - %s" % get_user_model().__name__,
        })

    def delete(self, request):
        o_id = request.body.decode('utf-8').split("=")[1]
        User = get_user_model()
        qs = User.objects.filter(id__exact=o_id).first()
        qs.delete()
        return self.render_to_response({})

    def datatable(self, request):
        User = get_user_model()
        qs = User.objects.all()
        defer = [
            'id',
            'full_name',
            'email',
            'phone_number',
            'is_active',
            'is_staff',
            'is_superuser']

        d = Datatable(request, qs, defer, key="id")
        return d.get_data()


class UserFormView(ProtectedMixin, TemplateView):
    template_name = "superuser/user.form.html"

    def get(self, request, *args, **kwargs):
        edit = request.GET.get("edit")
        User = get_user_model()

        if edit:
            form = UserForm(
                instance=User.objects.get(
                    id=edit), initial={
                    'password': ''})
        else:
            form = UserForm()

        return self.render_to_response({"form": form})

    def post(self, request):
        edit = request.GET.get("edit")
        User = get_user_model()

        if edit:
            u = User.objects.get(id=edit)
            u_pass = u.password
            form = UserForm(request.POST, instance=u, initial={'password': ''})
        else:
            form = UserForm(request.POST)

        if form.is_valid():
            user = form.save(commit=False)
            user.groups.set(form.cleaned_data.get('groups'))

            if form.cleaned_data.get(
                    'password') and form.cleaned_data.get('password') != '':
                user.set_password(form.cleaned_data.get('password'))
            else:
                user.password = u_pass

            user.save()
            if hasattr(user, 'permissions'):
                user.permissions.set(form.cleaned_data.get('permissions'))
                user.save()
            messages.success(
                request, _(
                    'User (%s) has been saved.' %
                    user.full_name))
            return redirect("superuser:user")
        else:
            return self.render_to_response({"form": form})



from __future__ import unicode_literals
import timeago
from datetime import timedelta
from itertools import chain
from django.db import models
from django.utils import timezone
from django.contrib.gis.db import models as geo
from django.contrib.gis.geos import Point
from django.contrib.sites.models import Site
from django.db.models import Manager as GeoManager
from django.contrib.contenttypes.models import ContentType
from django.conf import settings

from enterprise.libs.moment import to_timestamp
from enterprise.libs.base62 import base62_encode


User = settings.AUTH_USER_MODEL


class _BaseAbstract(models.Model):
    site = models.ForeignKey(Site, related_name="%(app_label)s_%(class)s_site",
                             blank=True, null=True, on_delete=models.CASCADE,)
    nonce = models.CharField(max_length=128, blank=True, null=True)
    id62 = models.CharField(
        max_length=100,
        db_index=True,
        blank=True,
        null=True)

    created_at = models.DateTimeField(db_index=True)
    created_at_timestamp = models.PositiveIntegerField(db_index=True)
    created_by = models.ForeignKey(User, on_delete=models.CASCADE,
                                   related_name="%(app_label)s_%(class)s_created_by")

    owned_at = models.DateTimeField(db_index=True)
    owned_at_timestamp = models.PositiveIntegerField(db_index=True)
    owned_by = models.ForeignKey(User, on_delete=models.CASCADE,
                                 related_name="%(app_label)s_%(class)s_owner")

    updated_at = models.DateTimeField(db_index=True, blank=True, null=True)
    updated_at_timestamp = models.PositiveIntegerField(
        db_index=True, blank=True, null=True)
    updated_by = models.ForeignKey(User, blank=True, null=True, on_delete=models.CASCADE,
                                   related_name="%(app_label)s_%(class)s_updated_by")

    published_at = models.DateTimeField(blank=True, null=True)
    published_at_timestamp = models.PositiveIntegerField(
        db_index=True, blank=True, null=True)
    published_by = models.ForeignKey(User, blank=True, null=True, on_delete=models.CASCADE,
                                     related_name="%(app_label)s_%(class)s_published_by")

    unpublished_at = models.DateTimeField(blank=True, null=True)
    unpublished_at_timestamp = models.PositiveIntegerField(
        db_index=True, blank=True, null=True)
    unpublished_by = models.ForeignKey(User, blank=True, null=True, on_delete=models.CASCADE,
                                       related_name="%(app_label)s_%(class)s_unpublished_by")

    approved_at = models.DateTimeField(blank=True, null=True)
    approved_at_timestamp = models.PositiveIntegerField(
        db_index=True, blank=True, null=True)
    approved_by = models.ForeignKey(User, blank=True, null=True, on_delete=models.CASCADE,
                                    related_name="%(app_label)s_%(class)s_approved_by")

    unapproved_at = models.DateTimeField(blank=True, null=True)
    unapproved_at_timestamp = models.PositiveIntegerField(
        db_index=True, blank=True, null=True)
    unapproved_by = models.ForeignKey(User, blank=True, null=True, on_delete=models.CASCADE,
                                      related_name="%(app_label)s_%(class)s_unapproved_by")

    deleted_at = models.DateTimeField(blank=True, null=True)
    deleted_at_timestamp = models.PositiveIntegerField(
        db_index=True, blank=True, null=True)
    deleted_by = models.ForeignKey(User, db_index=True, blank=True,
                                   null=True, on_delete=models.CASCADE,
                                   related_name="%(app_label)s_%(class)s_deleted_by")

    objects = GeoManager()

    def save(self, *args, **kwargs):
        now = timezone.now()

        # set Point
        [
            self.set_lat_lng(name, getattr(self, name))
            for name in self.get_all_field_names()
            if isinstance(self._meta.get_field(name), geo.PointField)
        ]

        # create first time record
        if self.created_at is None:
            self.site = Site.objects.get_current()
            self.created_at = now
            self.created_at_timestamp = to_timestamp(self.created_at)
            if self.created_by:
                self.owned_by = self.created_by
                self.owned_at = self.created_at
                self.owned_at_timestamp = self.created_at_timestamp

        # always update
        self.updated_at = now
        self.updated_at_timestamp = to_timestamp(self.updated_at)

        # save the first time record
        instance = super(_BaseAbstract, self).save(*args, **kwargs)

        # generate id62
        if self.id and not self.id62:
            self.id62 = base62_encode(self.id)
            kwargs['force_insert'] = False
            instance = super(_BaseAbstract, self).save(*args, **kwargs)

        return instance

    def approve(self, user=None, *args, **kwargs):
        if user:
            # mark when the record deleted
            self.unapproved_at = None
            self.unapproved_at_timestamp = None
            self.unapproved_by = None
            self.approved_at = timezone.now()
            self.approved_at_timestamp = to_timestamp(self.approved_at)
            self.approved_by = user

            # save it
            return super(_BaseAbstract, self).save(*args, **kwargs)

    def unapprove(self, user=None, *args, **kwargs):
        if user:
            # mark when the record reovered
            self.approved_at = None
            self.approved_at_timestamp = None
            self.approved_by = None
            self.unapproved_at = timezone.now()
            self.unapproved_at_timestamp = to_timestamp(self.unapproved_at)
            self.unapproved_by = user

            # save it
            return super(_BaseAbstract, self).save(*args, **kwargs)

    def reject(self, user=None, *args, **kwargs):
        self.unapprove(user, *args, **kwargs)

    def publish(self, user=None, *args, **kwargs):
        if user:
            # mark when the record deleted
            self.unpublished_at = None
            self.unpublished_at_timestamp = None
            self.unpublished_by = None
            self.published_at = timezone.now()
            self.published_at_timestamp = to_timestamp(self.published_at)
            self.published_by = user

            # save it
            return super(_BaseAbstract, self).save(*args, **kwargs)

    def unpublish(self, user=None, *args, **kwargs):
        if user:
            # mark when the record reovered
            self.published_at = None
            self.published_at_timestamp = None
            self.published_by = None
            self.unpublished_at = timezone.now()
            self.unpublished_at_timestamp = to_timestamp(self.unpublished_at)
            self.unpublished_by = user

            # save it
            return super(_BaseAbstract, self).save(*args, **kwargs)

    def delete(self, user=None, *args, **kwargs):
        if user:
            # mark when the record deleted
            self.deleted_by = user

        self.deleted_at = timezone.now()
        self.deleted_at_timestamp = to_timestamp(self.deleted_at)

        # save it if there's a deleter
        return super(_BaseAbstract, self).save(*args, **kwargs)

    def permanent_delete(self, *args, **kwargs):
        return super(_BaseAbstract, self).delete(*args, **kwargs)

    def undelete(self, user=None, *args, **kwargs):
        if user:
            # mark when the record undeleted
            self.deleted_at = None
            self.deleted_at_timestamp = None
            self.deleted_by = user

            # save it if there's a deleter
            return super(_BaseAbstract, self).save(*args, **kwargs)

    def log(self, user, message, *args, **kwargs):
        log = Log()
        log.content_type = self.get_content_type()
        log.object_id = self.id
        log.logged_by = user
        log.message = message
        log.save()

        if kwargs.get("mention"):
            log.mentions.add(kwargs.get("mention"))
        log.save()

        return log

    # Getter
    def get_owner(self):
        return self.owned_by

    def get_creator(self):
        return self.created_by_by

    def get_created_at(self):
        return {
            'utc': self.created_at,
            'timestamp': self.created_at_timestamp,
            'timeago': timeago.format(self.created_at.replace(tzinfo=None) + timedelta(hours=7))
        }

    def get_deleted_at(self):
        return {
            'utc': self.deleted_at,
            'timestamp': self.deleted_at_timestamp,
            'timeago': timeago.format(self.deleted_at.replace(tzinfo=None) + timedelta(hours=7))
        }

    def get_approved_at(self):
        return {
            'utc': self.approved_at,
            'timestamp': self.approved_at_timestamp,
            'timeago': timeago.format(self.approved_at.replace(tzinfo=None) + timedelta(hours=7))
        }

    def get_published_at(self):
        return {
            'utc': self.published_at,
            'timestamp': self.published_at_timestamp,
            'timeago': timeago.format(self.published_at.replace(tzinfo=None) + timedelta(hours=7))
        }

    def get_content_type(self):
        return ContentType.objects.get_for_model(self)

    def get_lat_lng(self, field_name):
        point = getattr(self, field_name, None)

        if point is not None and hasattr(point, "x") and hasattr(point, "y"):
            return {
                'latitude': point.y,
                'longitude': point.x
            }
        else:
            return {
                'latitude': 0,
                'longitude': 0
            }

    def get_all_sizes(self, field_name, sizes=None):
        if field_name in self._meta.get_all_field_names():
            image = getattr(self, field_name)

            if hasattr(image, "path"):
                return generate_all_sizes(image.path, sizes=sizes)

    def get_status(self):
        if not self.approved_by and self.unapproved_by:
            approve_message = "REJECTED"
        elif self.approved_by and not self.unapproved_by:
            approve_message = "APPROVED"
        else:
            approve_message = "waiting to be approved"

        if not self.published_by and self.unpublished_by:
            publish_message = "UNPUBLISHED"
        elif self.published_by and not self.unpublished_by:
            publish_message = "PUBLISHED"
        else:
            publish_message = "waiting to be published"

        return "Approval status: (%s), Publish status: (%s)" % (
            approve_message, publish_message)

    # Setter
    def set_lat_lng(self, field_name, value):
        point = None

        if hasattr(
                self, field_name) and value and "longitude" in value and "latitude" in value:
            point = Point(value["longitude"], value["latitude"])
            setattr(self, field_name, point)

        return point

    # Backward
    def get_all_field_names(self):
        return list(
            set(
                chain.from_iterable(
                    (field.name, field.attname) if hasattr(
                        field, 'attname') else (field.name,)
                    for field in self._meta.get_fields()
                    # For complete backwards compatibility, you may want to exclude
                    # GenericForeignKey from the results.
                    if not (field.many_to_one and field.related_model is None)
                )
            )
        )

    class Meta:
        abstract = True


class BaseModelGeneric(_BaseAbstract):
    created_by = models.ForeignKey(User, db_index=True, on_delete=models.CASCADE,
                                   related_name="%(app_label)s_%(class)s_created_by")

    class Meta:
        abstract = True


class BaseModelUnique(_BaseAbstract):
    created_by = models.OneToOneField(User, db_index=True, on_delete=models.CASCADE,
                                      related_name="%(app_label)s_%(class)s_created_by")

    class Meta:
        abstract = True



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: viewset.py
# Project: core.lakon.app
# File Created: Wednesday, 5th September 2018 1:23:29 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Wednesday, 5th September 2018 1:23:29 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Lakon, lakon.app
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''

from rest_framework import mixins
from enterprise.libs.rest_module.permission import JWTAuthenticated, IsAuthenticated, AllowAny
from rest_framework.viewsets import GenericViewSet
from .authentication import *


class PublicGenericViewSet(GenericViewSet):
    """
    Every authenticated user have access to the contents,
    but only owner has permission to update/delete
    """
    lookup_field = 'id62'

    def initialize_request(self, request, *args, **kwargs):
        su = super().initialize_request(
            request,
            *
            args,
            **kwargs)
        self.set_permissions()
        return su

    def set_permissions(self):
        """
        Instantiates and returns the list of permissions that this view requires.
        """
        if self.action in ['update', 'delete']:
            permission_classes = [IsOwnerAuthenticated]
        elif self.action in ['post']:
            permission_classes = [IsAuthenticated]
        else:
            permission_classes = [AllowAny]
        self.permission_classes = permission_classes
        return [permission() for permission in permission_classes]


class UGCGenericViewSet(GenericViewSet):
    """
    Every authenticated user have access to the contents,
    but only owner has permission to update/delete
    """
    lookup_field = 'id62'

    def initialize_request(self, request, *args, **kwargs):
        su = super().initialize_request(
            request,
            *
            args,
            **kwargs)
        self.set_permissions()
        return su

    def set_permissions(self):
        """
        Instantiates and returns the list of permissions that this view requires.
        """
        if self.action in ['update', 'delete']:
            permission_classes = [IsOwnerAuthenticated]
        else:
            permission_classes = [IsAuthenticated]
        self.permission_classes = permission_classes
        return [permission() for permission in permission_classes]


class PrivateContentGenericViewSet(GenericViewSet):
    """
    Only object owner have access to the contents
    """
    lookup_field = 'id62'

    def initialize_request(self, request, *args, **kwargs):
        su = super().initialize_request(request,*args,**kwargs)
        self.set_permissions()
        return su

    def set_permissions(self):
        """
        Instantiates and returns the list of permissions that this view requires.
        """
        if self.action in ['retrieve', 'update', 'delete']:
            permission_classes = [IsOwnerAuthenticated]
        else:
            permission_classes = [IsAuthenticated]
        self.permission_classes = permission_classes
        return [permission() for permission in permission_classes]

    def list(self, request):
        self.queryset = self.queryset.filter(
            owned_by=request.user
        ).order_by('-created_at')
        return super().list(request)


class JWTGenericViewSet(GenericViewSet):
    """
    View protected bt JWT
    """
    lookup_field = 'id62'

    def initialize_request(self, request, *args, **kwargs):
        su = super().initialize_request(request,*args,**kwargs)
        self.permission_classes = [JWTAuthenticated]
        return su


class UGCViewSet(UGCGenericViewSet,
                 mixins.CreateModelMixin,
                 mixins.RetrieveModelMixin,
                 mixins.UpdateModelMixin,
                 mixins.DestroyModelMixin,
                 mixins.ListModelMixin):
    pass


class PrivateContentViewSet(PrivateContentGenericViewSet,
                            mixins.CreateModelMixin,
                            mixins.RetrieveModelMixin,
                            mixins.UpdateModelMixin,
                            mixins.DestroyModelMixin,
                            mixins.ListModelMixin):
    pass


class PrivateContentRetrieveViewSet(PrivateContentGenericViewSet,
                                    mixins.RetrieveModelMixin,
                                    mixins.ListModelMixin):
    pass


class RetrieveViewSet(UGCGenericViewSet,
                      mixins.RetrieveModelMixin,
                      mixins.ListModelMixin):
    pass


class SingleObjectOwnedViewSet(UGCGenericViewSet,
                               mixins.RetrieveModelMixin,
                               mixins.UpdateModelMixin):
    pass



'''
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# File: nonce.py
# Project: core.lakon.app
# File Created: Friday, 14th September 2018 6:22:02 pm
#
# Author: Arif Dzikrullah
#         ardzix@hotmail.com>
#         https://github.com/ardzix/>
#
# Last Modified: Friday, 14th September 2018 6:22:02 pm
# Modified By: arifdzikrullah (ardzix@hotmail.com>)
#
# Hand-crafted & Made with Love
# Copyright - 2018 Lakon, lakon.app
# ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'''


class NonceObject(object):
    MODEL = None
    NONCE = None
    OBJ = None

    def __init__(self, *args, **kwargs):
        self.MODEL = kwargs.get("model")
        self.NONCE = kwargs.get("nonce")
        obj = self.MODEL.objects.filter(nonce=self.NONCE).first()
        if not obj:
            obj = self.MODEL(nonce=self.NONCE)
        self.OBJ = obj

    def get_instance(self):
        return self.OBJ

    def is_exist(self):
        if self.OBJ.id:
            return True
        else:
            return False
